#include "cGadgetString.h"
#include "../globalcode/datadir.h"

cGadgetString::cGadgetString ( )
{			

	iX = 0; iY = 0; iWidth = 1; iHeight = 1;	
	
	iCursorPosition = 0;
	
	TTF_Init ( );
	std::string file = get_datadir() + std::string("/bitstream_vera_sans.ttf");
	oFont = TTF_OpenFont (file.c_str(), 12 );
}

cGadgetString::~cGadgetString ( )
{
}

void cGadgetString::setRect ( SDL_Rect rect )
{
	iX = rect.x; iY = rect.y;
	iWidth = rect.w; iHeight = rect.h;
}

void cGadgetString::setDisplaySurface ( SDL_Surface* surface )
{
	oSurface = surface;
}

void cGadgetString::setActive ( )
{
	mainLoop ( );
}

void cGadgetString::setString ( string sData )
{
	if ( sData.size ( ) > 0 )
	{
		mText.clear ( );
		for ( unsigned int a = 0; a < sData.size ( ); a++ )
		{
			mText[ a ] = *sData.substr ( a, 1 ).c_str ( );
		}
		iCursorPosition = mText.size ( );
		refresh ( );
	}
}

string cGadgetString::getString ( )
{
	stringstream strstr;
	
	if ( mText.size ( ) )
	{		
		for ( unsigned int a = 0; a < mText.size ( ); a++ )
		{
			strstr << mText[ a ];
		}				
		
		return strstr.str ( );
	}
	else
	{
		return "";
	}
}

void cGadgetString::mainLoop ( )
{

	/*
		Enable unicode for SDL
	*/
	SDL_EnableUNICODE( 1 );	

	bool exitLoop = false;
	
	SDL_Event oEvent;
	
	int mousex, mousey;
	
	while ( !exitLoop )
	{
		mousex = SDL_GetMouseState ( &mousex, &mousey );
		
		while ( SDL_PollEvent ( &oEvent ) )
		{}
		
		switch ( oEvent.type )
		{
			case SDL_MOUSEBUTTONDOWN:								
				
				if ( mousex < iX || mousey < iY || mousex > iX + iWidth || mousey > iY + iHeight )
				{
					exitLoop = true;
				}				
				
				break;
						
			case SDL_KEYDOWN:								
				
				if ( oEvent.key.keysym.sym == SDLK_LEFT )
				{
					if ( iCursorPosition > 0 )
						iCursorPosition --;
				}
				else if ( oEvent.key.keysym.sym == SDLK_RIGHT )
				{
					if ( iCursorPosition < ( int )mText.size ( ) )
						iCursorPosition ++;						
				}
				else if ( oEvent.key.keysym.sym == SDLK_BACKSPACE )
				{
					map < int, char > oText;
					for ( int a = 0; a < ( int )mText.size ( ); a++ )
					{
						if ( a != iCursorPosition - 1 )
						{
							oText[ ( int )oText.size ( ) ] = mText[ a ];
						}
					}
					mText = oText;
					if ( iCursorPosition > 0 )
						iCursorPosition --;
				}
				else if ( oEvent.key.keysym.sym == SDLK_DELETE )
				{
					map < int, char > oText;
					for ( int a = 0; a < ( int )mText.size ( ); a++ )
					{
						if ( a != iCursorPosition + 1 )
						{
							oText[ ( int )oText.size ( ) ] = mText[ a ];
						}
					}
					mText = oText;
				}
				else if ( oEvent.key.keysym.sym == SDLK_RETURN )
				{
					exitLoop = true;
				}
				else if ( oEvent.key.keysym.sym == SDLK_ESCAPE )
				{
					exitLoop = true;
				}
				else
				{										
					map < int, char > tempString;										
					
					if ( oEvent.key.keysym.unicode < 0xFF & oEvent.key.keysym.unicode > 0 )
					{
						int intChar = ( int )oEvent.key.keysym.unicode;
						if ( oEvent.key.keysym.mod & KMOD_SHIFT ) 
							intChar = toupper( ( int )oEvent.key.keysym.unicode );					
											
						if ( mText.size ( ) )
						{
							for ( int a = 0; a < ( int )mText.size ( ); a++ )
							{							
								tempString[ tempString.size ( ) ] = mText[ a ];							
								if ( a == iCursorPosition - 1 )
								{
									tempString[ tempString.size ( ) ] = ( char )intChar;
								}
							}
							iCursorPosition ++;
						}
						else
						{
							tempString[ 0 ] = ( char )intChar;
							iCursorPosition ++;
						}
											
						mText = tempString;
					}
				}
												
				refresh ( );
				
				bool keyUp = false;
					
				while ( !keyUp )
				{
					SDL_PollEvent ( &oEvent );					
					switch ( oEvent.type )
					{
						case SDL_KEYUP:
							keyUp = true;
							break;
						default: break;
					}
					SDL_Delay ( 2 );
				}				
				
				break;
		}
		SDL_Delay ( 2 );		
	}	
	
	refresh ( );
}

void cGadgetString::refresh ( )
{

	/*
		Make the rect
	*/
	
	SDL_Rect tmpRect = { iX, iY, iWidth, iHeight };	
	Uint32 colorW = SDL_MapRGBA ( oSurface->format, 200, 200, 200, 255 );
	SDL_FillRect ( oSurface, &tmpRect, colorW );		
			
	/*
		Make 3d illusion
	*/
	for ( int a = iX; a < iX + iWidth - 1; a++ )
	{
		putSDLPixel ( a, iY, SDL_MapRGBA ( oSurface->format, 100, 100, 100, 255 ), oSurface );	
		putSDLPixel ( iX + iWidth - ( a - iX + 1 ), iY + iHeight - 1, SDL_MapRGBA ( oSurface->format, 255, 255, 255, 255 ), oSurface );
	}
	for ( int b = iY; b < iY + iHeight - 1; b++ )
	{
		putSDLPixel ( iX, b, SDL_MapRGBA ( oSurface->format, 100, 100, 100, 255 ), oSurface );	
		putSDLPixel ( iX + iWidth - 1, b + 1, SDL_MapRGBA ( oSurface->format, 255, 255, 255, 255 ), oSurface );
	}
	
	/*
		Render current text string
	*/
	
	SDL_Color colorB = { 0, 0, 0, 255 };
	
	stringstream stri; 
	
	bool cursorDrawn = false;
	
	if ( mText.size ( ) )
	{
		for ( int a = 0; a < ( int )mText.size ( ); a++ )
		{					
			if ( a == iCursorPosition )
			{				
				stri << '>';
				cursorDrawn = true;
			}
			stri << mText[ a ];
		}	
	}
	if ( !cursorDrawn )
		stri << '>';
	
	SDL_Surface* textSurf = TTF_RenderText_Blended ( oFont, stri.str ( ).c_str ( ), colorB );		
	SDL_Rect rect = { iX, iY + ( int )( ( double )iHeight / 2 ) - ( int )( ( double )textSurf->h / 2 ), textSurf->w, textSurf->h };
	SDL_BlitSurface ( textSurf, NULL, oSurface, &rect );				
	SDL_FreeSurface ( textSurf );				
		
	SDL_UpdateRect ( oSurface, iX, iY, iWidth, iHeight );		
	
}
