	
	#include "cFileIO.h"	
	
	/* Constructor */
	cFileIO::cFileIO ( )
	{
		this->fileName = "";
	}
	
	/* Destructor */
	cFileIO::~cFileIO ( )
	{
		this->fileName = "";
	}		
	
	/* Set a filename */
	void cFileIO::setFilename ( string filename )
	{
		this->fileName = filename;
	}		
	
	/* Check if we have a filename */
	bool cFileIO::getFilename ( )
	{
		if ( this->fileName.size ( ) )
			return true;
		else
			return false;
	}
	
	/* Load a file */
	bool cFileIO::importFile ( cImage* image )
	{
		/* TODO: add error checker if file exists and is png! */		
		SDL_Surface* tempSurface;
		tempSurface = IMG_Load ( this->fileName.c_str ( ) );		
		if( !tempSurface )
		{
    		printf("IMG_Load: %s\n", IMG_GetError());
    		return false;
    	}
    	else
    	{
    		cImageColor tempCol;
    		SDL_Rect rect;    		
			rect.w = tempSurface->w;
			rect.h = tempSurface->h;
			rect.x = 0; rect.y = 0;
			int a, b;
			for ( a = 0; a < image->getWidth ( ); a++ )
				for ( b = 0; b < image->getHeight ( ); b++ )
					if ( a < rect.w & b < rect.h )
					{
						tempCol = image->getPixel ( a, b, tempSurface );
						image->putPixel ( a, b, &tempCol );
					}
			return true;
		}
	}
			
	/* Do the magic of exporting into PNG file in here. */
	bool cFileIO::exportFile  ( SDL_Surface* surf, string FORMAT )
	{
		if ( FORMAT == "PNG" )
		{
			png_structp png_ptr;
			png_infop   info_ptr;
			png_byte**  row_ptrs;
			ofstream    stream;
			int         i;
			
			// Create a write struct.
			if ( !(png_ptr = png_create_write_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL ) ) )	
				return false;
			
			// Create an info struct.
			if ( !( info_ptr = png_create_info_struct( png_ptr ) ) )
			{
				// Free the unused write struct.
				png_destroy_write_struct ( &png_ptr, ( png_infopp )NULL );
				return false;
			}
			// Create a stream.
			stream.open ( this->fileName.c_str(), ios::trunc );
			if ( !stream.is_open() )
				return false;
	
			// Register the write function.
			png_set_write_fn ( png_ptr, (ostream* ) &stream, png_user_write, png_user_flush );
			
			// Create the info header.
			png_byte coltype;
			switch ( surf->format->BytesPerPixel )
			{			
				case 1: 
					coltype = PNG_COLOR_TYPE_GRAY; 
					break;
				case 3: 
					coltype = PNG_COLOR_TYPE_RGB; 
					break;
				case 4: 
					coltype = PNG_COLOR_TYPE_RGB_ALPHA; 
					break;
				default:
					return false;
			}
			png_set_IHDR (png_ptr, info_ptr, surf->w, surf->h, 8, coltype,
				PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
				PNG_FILTER_TYPE_DEFAULT );
			
			// Write the info header.
			png_write_info ( png_ptr, info_ptr );
			
			// Allocate row pointers.
			if ( !( row_ptrs = new png_byte*[surf->h] ) )
			{
				png_destroy_write_struct ( &png_ptr, &info_ptr );
				return false;
			}
			
			// Build the row pointers.
			for ( i = 0; i < surf->h ; i++ )
				row_ptrs[i] = ( png_byte* )surf->pixels + i * surf->pitch;
			
			// Write the image data.
			png_write_image( png_ptr, row_ptrs );
			png_write_end( png_ptr, info_ptr );
			png_destroy_write_struct( &png_ptr, &info_ptr ); 
			
			delete[] row_ptrs;				
			
			return true;
		}
		else
			return false;
	}
		
	// Saving
	bool cFileIO::save ( )
	{		
		cImage image;
		int stringsize = 0;	
		FILE* outFile;		
		stringstream outStringData; 
		
		outFile = fopen ( this->fileName.c_str ( ), "w+" );
		
		if ( !outFile )
		{			
			return false;
		}
																			
		outStringData << "Lunapaint data format v1.0|Imagename|";
		outStringData << this->width << "|" << this->height << "|" << this->layers << "|" << this->frames << "|";				
						
		// Basic size
		stringsize = outStringData.str ( ).size ( );				
				
		fwrite ( outStringData.str ( ).c_str ( ), stringsize, 1, outFile );				
																
		for ( int a = 0; a < this->layers * this->frames; a++ )
		{			
			fwrite ( 
				this->surfaces[ a ]->pixels, 
				this->surfaces[ a ]->w * this->surfaces[ a ]->format->BytesPerPixel, 
				this->surfaces[ a ]->h, 
				outFile 
			);								
		}										
		
		fclose ( outFile );
																
		return true;
	}
	
	bool cFileIO::load ( )
	{	
		FILE* stream;
		long size;
				
		stream = fopen ( this->fileName.c_str ( ), "r" );
		
		if ( !stream )
			return false;
			
		// Get the size
		size = sizeof ( stream );
		
		string tempData; 
		int mode = 0; 
		bool breakout = false;
		
		int a; int reg = 0;		
		
		while ( breakout == false )
		{		
			char inData;

			fread ( ( void* )&inData, sizeof ( char ), 1, stream );				
				
			if ( inData == '|' )
			{
				mode++;
				reg = 1;
			}
			else
			{
				tempData += inData;
				a++;
			}
				
			if ( reg == 1 )
			{
				switch ( mode )
				{
					case 0:
						break;
					case 1:
						break;
					case 2: 
						this->fileName = tempData; 
						tempData = "";
						break;
					case 3:
						this->width = atoi ( tempData.c_str ( ) );
						tempData = "";
						break;
					case 4:
						this->height = atoi ( tempData.c_str ( ) );
						tempData = "";
						break;
					case 5:
						this->layers = atoi ( tempData.c_str ( ) );
						tempData = "";
						break;
 					case 6:
						this->frames = atoi ( tempData.c_str ( ) );
						tempData = "";			
						breakout = true;								
						break;
					default:						
						// We never get here						
						break;					
				}
				reg = 0;
			}			
		}	
		
		// Read in the image data		
		
		/* To hold the masks */
		Uint32 rmask, gmask, bmask, amask;		
		
		if ( SDL_BYTEORDER == SDL_BIG_ENDIAN )
		{
			rmask = 0xff000000;
			gmask = 0x00ff0000;
			bmask = 0x0000ff00;
			amask = 0x000000ff;
		}
		else
		{
			rmask = 0x000000ff;
			gmask = 0x0000ff00;
			bmask = 0x00ff0000;
			amask = 0xff000000;
		}	
		
		for ( int a = 0; a < this->layers * this->frames; a++ )
		{							
			this->surfaces[ a ] = SDL_CreateRGBSurface ( SDL_SWSURFACE, this->width, this->height, 32, rmask, gmask, bmask, amask );						
			fread ( this->surfaces[ a ]->pixels, this->surfaces[ a ]->format->BytesPerPixel * this->width, this->height, stream );																
		}		
		fclose ( stream );	
				
		return true;
	}

	void cFileIO::setFrames ( int number )
	{
		this->frames = number;
		this->initialized = false;
	}
	
	void cFileIO::setLayers ( int number )
	{
		this->layers = number;	
		this->initialized = false;
	}
	
	void cFileIO::setWidth ( int number )
	{
		this->width = number;		
	}
	
	void cFileIO::setHeight ( int number )
	{
		this->height = number;		
	}
	
	void cFileIO::copySurface ( SDL_Surface* surface, int number )
	{
		if ( !this->initialized )
		{
			this->initialized = true;
			this->surfaces.clear ( );			
		}
		this->surfaces[ number ] = surface;
	}
	
	int cFileIO::getWidth ( )
	{
		return this->width;
	}
	
	int cFileIO::getHeight ( )
	{
		return this->height;
	}
	
	int cFileIO::getFrames ( )
	{
		return this->frames;
	}
	
	int cFileIO::getLayers ( )
	{
		return this->layers;
	}
	
	/* Fast way to copy whole pixels :-D */
	void cFileIO::copyFrameBuffer ( int frameoffset, SDL_Surface* surface )
	{		
		surface->pixels = this->surfaces[ frameoffset ]->pixels;		
	}
