#include "cImage.h"

cImage::cImage ( )
{		
	iWidth = 0;
	iHeight = 0;
	iFramesnum = 10;
	iCurrentFrame = 0;
}
	
cImage::~cImage ( )
{
	iWidth = 0;
	iHeight = 0;
}

void cImage::setTempBuffer ( SDL_Surface* surface )
{
	this->temp = surface; 
}		

int cImage::resize ( int width, int height )
{												
	iWidth = width; iHeight = height;
	
	/* To hold the masks */
	Uint32 rmask, gmask, bmask, amask;
	
	if ( SDL_BYTEORDER == SDL_BIG_ENDIAN )
	{
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
	}
	else
	{
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
	}
			
	SDL_Rect rect; rect.x = 0; rect.y = 0; rect.w = iWidth; rect.h = iHeight;
	
	for ( int a = 0; a < iFramesnum; a++ )
	{
		mFrames[ a ] = SDL_CreateRGBSurface ( SDL_SWSURFACE, iWidth, iHeight, 32, rmask, gmask, bmask, amask );			
		Uint32 col = SDL_MapRGBA ( mFrames[ 0 ]->format, 100, 100, 100, 0 );
		SDL_FillRect ( mFrames[ a ], &rect, col );
	}		
	return 0;
}

void cImage::putRawPixel ( int x, int y, Uint32 pixel )
{			
	if ( x >= 0 & x < iWidth & y >= 0 & y < iHeight )
	{				
		*( ( Uint32* )mFrames[ iCurrentFrame ]->pixels + y * mFrames[ iCurrentFrame ]->pitch / 4 + x ) = pixel;																		
	}
}		

void cImage::putRawPixel ( int x, int y, Uint32 pixel, SDL_Surface* surface )
{			
	if ( x >= 0 & x < iWidth & y >= 0 & y < iHeight )
	{				
		*( ( Uint32* )surface->pixels + y * surface->pitch / 4 + x ) = pixel;																		
	}
}		

/* Get a raw pixel for quit processing */
Uint32 cImage::getRawPixel ( int x, int y )
{	
	return *( ( Uint32* )mFrames[ iCurrentFrame ]->pixels + y * mFrames[ iCurrentFrame ]->pitch / 4 + x );
}

/* Get a raw pixel for quit processing */
Uint32 cImage::getRawPixel ( int x, int y, SDL_Surface* surface )
{				
	return *( ( Uint32* )surface->pixels + y * surface->pitch / 4 + x );
}		

// Clear the image canvas 
void cImage::clear ( )
{		
	this->empty ( );
}

/* empty all layers */
void cImage::empty ( )
{	
	Uint32 pixel = SDL_MapRGBA ( mFrames[ iCurrentFrame ]->format, 100, 100, 100, 0 );		
	for ( int x = 0; x < iWidth; x++ )
	{
		for ( int y = 0; y < iHeight; y++ )
		{
			this->putRawPixel ( x, y, pixel );
		}
	}
}		

void cImage::putPixel ( int x, int y, cImageColor* color )
{		
	if ( x >= 0 & x < iWidth & y >= 0 & y < iHeight )
	{	
		// Quickly add the pixel										
		*( (Uint32 *)mFrames[ iCurrentFrame ]->pixels + y * mFrames[ iCurrentFrame ]->pitch / 4 + x ) = 
			SDL_MapRGBA ( mFrames[ iCurrentFrame ]->format, color->r, color->g, color->b, color->a );				
	}		
}

void cImage::putPixel ( int x, int y, cImageColor color )
{				
	if ( x >= 0 & x < iWidth & y >= 0 & y < iHeight )
	{			
		// Quickly add the pixel
		*( ( Uint32* )mFrames[ iCurrentFrame ]->pixels + y * mFrames[ iCurrentFrame ]->pitch / 4 + x ) =
			SDL_MapRGBA ( mFrames[ iCurrentFrame ]->format, color.r, color.g, color.b, color.a );				
	}		
}

void cImage::putPixel ( int x, int y, cImageColor* color, SDL_Surface* surface )
{		
	if ( x >= 0 & x < surface->w & y >= 0 & y < surface->h )
	{					
		// Quickly blit
		*( ( Uint32* )surface->pixels + y * surface->pitch / 4 + x ) = 
			SDL_MapRGBA ( surface->format, color->r, color->g, color->b, color->a );
	}		
}		

/* Add value to a pixel -- stack the color value! */
void cImage::addPixel ( int x, int y, cImageColor* color )
{		
	cImageColor source = getPixel ( x, y );		

	// Here, mix colours including alpha
	if ( source.a > 0 )
	{
		if ( source.a < 255 )
		{																		
			color->r -= ( int )( ( color->r - source.r ) / 255.0 * source.a );
			color->g -= ( int )( ( color->g - source.g ) / 255.0 * source.a );
			color->b -= ( int )( ( color->b - source.b ) / 255.0 * source.a );
			
			if ( color->a + source.a <= 255 )
				color->a += source.a;
			else
				color->a = 255;
		}
		else
		{
			color->r = source.r;
			color->g = source.g;
			color->b = source.b;
			color->a = source.a;
		}				
	}
}		

/* Add value to a pixel -- stack the color value! */
void cImage::addPixel ( int x, int y, cImageColor* color, SDL_Surface* destSurface )
{		
	cImageColor source = this->getPixel ( x, y, destSurface );		
				
	if ( source.a > 0 )
	{																							
		color->r -= ( int )( ( color->r - source.r ) / 255.0 * source.a );
		color->g -= ( int )( ( color->g - source.g ) / 255.0 * source.a );
		color->b -= ( int )( ( color->b - source.b ) / 255.0 * source.a );
		
		if ( color->a + source.a <= 255 )
			color->a += source.a;
		else
			color->a = 255;							
	}
}

/* Add value to a pixel -- stack the color value! */
cImageColor cImage::addPixel ( int x, int y, cImageColor color )
{		
	cImageColor source = this->getPixel ( x, y );		
							
	// Here, mix colours including alpha
	if ( source.a > 0 )
	{
		if ( source.a < 255 )
		{																		
			color.r -= ( int )( ( color.r - source.r ) / 255.0 * source.a );
			color.g -= ( int )( ( color.g - source.g ) / 255.0 * source.a );
			color.b -= ( int )( ( color.b - source.b ) / 255.0 * source.a );				
			
			if ( color.a + source.a <= 255 )
				color.a += source.a;
			else
				color.a = 255;												
		}
		else
		{			
			color.r = source.r;
			color.g = source.g;
			color.b = source.b;
			color.a = source.a;
		}				
	}
	
	return color;
}

/* Add value to a pixel -- stack the color value! */
cImageColor cImage::addPixel ( int x, int y, cImageColor color, SDL_Surface* destSurface )
{		
	cImageColor source = this->getPixel ( x, y, destSurface );		
				
	if ( source.a > 0 )
	{	
		// Make the pixel more transparent																									
		color.r -= ( int )( ( color.r - source.r ) / 255.0 * source.a );
		color.g -= ( int )( ( color.g - source.g ) / 255.0 * source.a );
		color.b -= ( int )( ( color.b - source.b ) / 255.0 * source.a );
															
		if ( color.a + source.a <= 255 )
			color.a += source.a;
		else
			color.a = 255;	
	}
	return color;
}		

// Blend two colours
void cImage::combinePixels ( cImageColor* color, cImageColor source )
{					
	source.r -= ( int )( ( source.r - color->r ) / 255.0 * color->a );
	source.g -= ( int )( ( source.g - color->g ) / 255.0 * color->a );
	source.b -= ( int )( ( source.b - color->b ) / 255.0 * color->a );
	
	*color = source;		
}

void cImage::nextFrame ( )
{						
	if ( iCurrentFrame + 1 < iFramesnum )
		iCurrentFrame++;
	else
		iCurrentFrame = 0;
}

void cImage::prevFrame ( )
{
	if ( iCurrentFrame - 1 >= 0 )
		iCurrentFrame--;
	else
		iCurrentFrame = iFramesnum - 1;
}

// Sets the number of mFrames (be careful!!!)
void cImage::setFrames ( int num )
{	
	iFramesnum = num;		
}

// Return how many mFrames we have!
int cImage::getFrames ( )
{
	return iFramesnum;
}

// Goto first frame
void cImage::rewind ( )
{
	iCurrentFrame = 0;
}


/* Return a pointer to this buffer */
SDL_Surface* cImage::getBuffer ( )
{		
	return mFrames[ iCurrentFrame ];		
}	

// Return the next frame buffer.... - if the next is empty give the first.
SDL_Surface* cImage::getNextFrameBuffer ( )
{
	if ( iCurrentFrame + 1 < iFramesnum )
		return mFrames[ iCurrentFrame + 1 ];
	else
		return mFrames[ 0 ];
}

// Return the prev frame buffer..... - if the prev is empty, give the last
SDL_Surface* cImage::getPrevFrameBuffer ( )
{
	if ( iCurrentFrame - 1 >= 0 )
		return mFrames[ iCurrentFrame - 1 ];
	else		
		return mFrames[ iFramesnum - 1 ];
}

int cImage::getWidth ( )
{
	return ( int )mFrames[ 0 ]->w;
}

int cImage::getHeight ( )
{
	return ( int )mFrames[ 0 ]->h;
}

int cImage::getCurrentFrame ( )
{
	return iCurrentFrame;
}	
