#ifndef _CIMAGE_H_
#define _CIMAGE_H_

#include <iostream>
#include <map>
#include <string>
#include <SDL/SDL.h>

using namespace std;

struct cImageColor
{
	int r, g, b, a;		
};

struct cCoords
{
	int x, y, w, h;
	bool Activity;
};
	
class cImage
{
	private:												
		/* Default width and height */
		int iWidth, iHeight;			
		/* Default layer number */
		int iCurrentLayer;				
		int iCurrentFrame;	
		int iFramesnum; // Number of frames	
		map < int, SDL_Surface* > mFrames;			
	
	public:				
		cImage ( );
		~cImage ( ); 			
		
		/* The temp buffer for temporary writes */
		SDL_Surface* temp;						
		
		int resize ( int width, int height );
		
		/* Other */
		int getHeight ( );
		int getWidth ( );
		void clear ( );
		void empty ( ); 									
		
		SDL_Surface* getBuffer ( );
		SDL_Surface* getNextFrameBuffer ( );
		SDL_Surface* getPrevFrameBuffer ( );
		
		void setTempBuffer ( SDL_Surface* surface );
		
		void combinePixels ( cImageColor* color, cImageColor source );
		
		void putPixel ( int x, int y, cImageColor color );
		void putPixel ( int x, int y, cImageColor* color );			
		void putPixel ( int x, int y, cImageColor* color, SDL_Surface* surface );
		
		inline cImageColor getPixel ( int x, int y );
		inline cImageColor getPixel ( int x, int y, SDL_Surface* surface );												
		
		void addPixel ( int x, int y, cImageColor* color );			
		void addPixel ( int x, int y, cImageColor* color, SDL_Surface* destSurface );
		
		cImageColor addPixel ( int x, int y, cImageColor color );			
		cImageColor addPixel ( int x, int y, cImageColor color, SDL_Surface* destSurface );						
		
		void putRawPixel ( int x, int y, Uint32 pixel );			
		void putRawPixel ( int x, int y, Uint32 pixel, SDL_Surface* surface );
		Uint32 getRawPixel ( int x, int y );			
		Uint32 getRawPixel ( int x, int y, SDL_Surface* surface );		
		
		void nextFrame ( );
		void prevFrame ( );
		void rewind ( );
		void setFrames ( int num );
		int getCurrentFrame ( );
		int getFrames ( );			
};

/* Get the visible pixel - start with the foremost pixel */
inline cImageColor cImage::getPixel ( int x, int y )
{								
	if ( x < iWidth & y < iHeight & x >= 0 & y >= 0 )
	{						
		/* Quickly get the pixel */
		Uint32 resPixel = *( ( Uint32 * )mFrames[ iCurrentFrame ]->pixels + y * mFrames[ iCurrentFrame ]->pitch / 4 + x );
		
		/* Setup and return the actual color values */
		
		Uint8 r, g, b, a;			
		
		SDL_GetRGBA ( resPixel, mFrames[ iCurrentFrame ]->format, &r, &g, &b, &a );			
		
		if ( a > 0 )
		{														
			cImageColor col;
			col.r = ( int )r;
			col.g = ( int )g;
			col.b = ( int )b;
			col.a = ( int )a;								
			return col;
		}
	}
	
	cImageColor col;																					
	col.r = 0;
	col.g = 0;
	col.b = 0;
	col.a = 0;	
	return col;
}	

/* Get the visible pixel - start with the foremost pixel */
inline cImageColor cImage::getPixel ( int x, int y, SDL_Surface* surface )
{								
	if ( x < surface->w & y < surface->h & x >= 0 & y >= 0 )
	{																		
		/* Quickly fetch the pixel */
		Uint32 resPixel = *( (Uint32 *)surface->pixels + y * surface->pitch / 4 + x );
		
		/* Setup and return the actual color values */
		
		Uint8 r, g, b, a;			
		
		SDL_GetRGBA ( resPixel, surface->format, &r, &g, &b, &a );					
					
		if ( a > 0 )
		{
			cImageColor col;																					
			col.r = ( int )r;
			col.g = ( int )g;
			col.b = ( int )b;
			col.a = ( int )a;					
			return col;				
		}
	}		
		
	cImageColor col;																					
	col.r = 0;
	col.g = 0;
	col.b = 0;
	col.a = 0;			
	return col;
}

#endif
