#include "cToolkit.h"		
#include "../globalcode/datadir.h"

cToolkit::cToolkit ( int argLayers )
{			
	/*
		Initialize the TTF library
	*/
	
	TTF_Init ( );

        std::string file = get_datadir() + std::string("/bitstream_vera_sans.ttf");
 	oSmallFont = TTF_OpenFont ( file.c_str(), 8 );				
	
	/*
		Set up variables for use in this object:		
	*/

	iPrevX = -1; // Previously set mouse coordinate X
	iPrevY = -1; // Previously set mouse coordinate Y
	
	iMouseCoordX = -1; // Current mouse coordinate X
	iMouseCoordY = -1; // Current mouse coordinate Y
	iCurrentFrame = 0; // Current frame
	
	quitDirective = false; // Are we going to quit?
	bPanelsHidden = false;	// Are the panels hidden?					
	iCurrentLayer = 1; // The current layer
	
	oActiveCanvas = &mCanvas[ iCurrentLayer ];			
	
	sFileName = "";		
	bUsingSwap = false;								
	bFullscreen = false;			
	
	iLayerNum = argLayers;	
	
	iOnionSkin = 0;	
	
	bUsingGrid = false;		
	iGridSize = 8;		
	
	oLayers.setLayers ( iLayerNum - 3 );	
	
	for ( int a = 0; a < iLayerNum - 1; a++ )
		mCanvas[ a ].setTempBuffer ( oTempBuffer );		
	
	canvasOffsetX = 0;
	
	canvasOffsetY = 0;
	
	canvasZoom = 1;
								
	sUsingCursor = "none";
	
	if ( SDL_BYTEORDER == SDL_BIG_ENDIAN )
	{
		RMASK = 0xff000000;
		GMASK = 0x00ff0000;
		BMASK = 0x0000ff00;
		AMASK = 0x000000ff;
	}
	else
	{
		RMASK = 0x000000ff;
		GMASK = 0x0000ff00;
		BMASK = 0x00ff0000;
		AMASK = 0xff000000;
	}	
	
	/* Slider for tool draw power */
	oPowerSlider.setButtonRect ( ( SDL_Rect ){ 16, 0, 32, 16 } );
	oPowerSlider.setSliderRect ( ( SDL_Rect ){ 16, 0, 606, 16 } );
	oPowerSlider.setScope ( 0, 255 );
	oPowerSlider.setValue ( 255 );
	oPowerSlider.setDirection ( "x" );
}

cToolkit::~cToolkit ( ){}		

/* 
	Select output for this image ( window ):
*/
void cToolkit::setOutput ( SDL_Surface* window, int width, int height )
{
	oWindow = window;
	iDisplayBPP = oWindow->format->BytesPerPixel;
	iWindowWidth = width; iWindowHeight = height;	
	displayPTRpixels = (Uint8 *)oWindow->pixels;
	displayPitch = oWindow->pitch;
	
	oFileDialogue.setOutputSurface ( window );		
	oPowerSlider.setDisplaySurface ( window );
}

void cToolkit::Action ( )
{	
	/*
		Calculate canvas skew:
	*/
	calculateSkew ( );			

	/* 
		Just initialize the normal cursor:
	*/
	if ( sUsingCursor == "none" )
	{
		setCursor ( mouseArrow );
		sUsingCursor = "normal";
		
		/* 
			We start off with the pen tool:
		*/
		changeTool ( "pen" );							
	}
	
	/*
		Fetch events and execute their routines:			
	*/
	while ( SDL_PollEvent ( &eEvent ) )
	{									
		switch ( eEvent.type )
		{
			case SDL_MOUSEBUTTONDOWN:							
				if ( !bPanelsHidden	)
				{
					if ( oPowerSlider.isMouseOver ( iMouseCoordX, iMouseCoordY ) )
					{
						oPowerSlider.setActive ( );
						oBrush.setPower ( ( int )oPowerSlider.getValue ( ) );
					}
					else if ( iMouseCoordX > oWindow->w - 20 )
					{						
						oPalette.setColor ( iMouseCoordY );						
						oColorBuffer= oPalette.getColor ( );
						oBrush.setColorPointer ( oPalette.getColorPointer ( ) );
						updateScreen ( );						
					}
					else if ( iMouseCoordY >= 0 && iMouseCoordY < 144 && iMouseCoordX >= 0 && iMouseCoordX < 18 )
					{
						string directive = oTools.setTool ( iMouseCoordY );
						
						if ( directive == "pen" )
							changeTool ( "pen" );
						else if ( directive == "line" )
							changeTool ( "line" );
						else if ( directive == "fill" )
							changeTool ( "fill" );
						else if ( directive == "clipbrush" )
							changeTool ( "clipbrush" );
						else if ( directive == "rectangle" )
							changeTool ( "rectangle" );
						else if ( directive == "circle" )
							changeTool ( "circle" );
						else if ( directive == "colorpicker" )
							changeTool ( "colorpicker" );						
						else if ( directive == "clear" )
						{
							mCanvas[ iCurrentLayer ].empty ( );
							drawLayer ( );
						}
					}
					else if ( iMouseCoordY > ( oWindow->h - ( oWindow->h / 2 ) ) & iMouseCoordX < 20 & bPanelsHidden == false )
					{						
						iCurrentLayer = oLayers.getLayer ( iMouseCoordY );
						bUsingSwap = false;
						oActiveCanvas = &mCanvas[ iCurrentLayer ];	
						if ( iOnionSkin )											
						{
							drawLayer ( );
						}
						else
						{
							updateScreen ( );
						}
					}
					else
					{
						ptrTool->setMouseMode ( "on" );						
					}
				}
				else
					ptrTool->setMouseMode ( "on" );						
				break;
			case SDL_QUIT:
				quitDirective = true;
				break;
			case SDL_MOUSEBUTTONUP:
				ptrTool->setMouseMode ( "off" );										
				break;
			case SDL_KEYDOWN:
				switch ( eEvent.key.keysym.sym )
				{
					case SDLK_ESCAPE:
						quitDirective = true;
						break;		
					case SDLK_KP_PLUS:							
						ptrTool->sendQuery ( "sizePlus" );
						oBrush.sendQuery ( "sizePlus" );
						break;
					case SDLK_KP_MINUS:
						ptrTool->sendQuery ( "sizeMinus" );
						oBrush.sendQuery ( "sizeMinus" );
						break;
					case SDLK_1:
						/* 
							No frames on swap..., just remember that...
						*/
						if ( !bUsingSwap )
						{
							for ( int a = 1; a < iLayerNum - 2; a++ )
								mCanvas[ a ].prevFrame ( );								
						}
						drawLayer ( );
						break;	
					case SDLK_2:
						// No frames on swap...
						if ( !bUsingSwap )
						{
							for ( int a = 1; a < iLayerNum - 2; a++ )
								mCanvas[ a ].nextFrame ( );								
						}
						drawLayer ( );
						break;	
					case SDLK_PAGEUP:							
						changeZoom ( 1, iMouseCoordX, iMouseCoordY );
						drawLayer ( );
						break;
					case SDLK_PAGEDOWN:
						changeZoom ( -1, 0, 0 );
						drawLayer ( );
						break;						
					case SDLK_LEFT:
						moveZoom ( 1 );
						drawLayer ( );							
						break;
					case SDLK_RIGHT:
						moveZoom ( 2 );
						drawLayer ( );
						break;
					case SDLK_UP:
						moveZoom ( 3 );
						drawLayer ( );
						break;
					case SDLK_DOWN:
						moveZoom ( 4 );
						drawLayer ( );
						break;
					case SDLK_RETURN:
						
						/* 
							Toggle fullscreen flag:
						*/
						
						if ( bFullscreen )
						{
							oWindow = SDL_SetVideoMode ( oWindow->w, oWindow->h, iDisplayBPP * 8, SDL_SWSURFACE );
							bFullscreen = false;
						}
						else
						{
							oWindow = SDL_SetVideoMode ( oWindow->w, oWindow->h, iDisplayBPP * 8, SDL_SWSURFACE|SDL_FULLSCREEN );
							bFullscreen = true;																
						}				
									
						oFileDialogue.setOutputSurface ( oWindow );
						oPowerSlider.setDisplaySurface ( oWindow );
						
						drawLayer ( );
						
						break;
					case SDLK_m:
						if ( eEvent.key.keysym.mod == KMOD_LCTRL || eEvent.key.keysym.mod == KMOD_RCTRL )
						{
							mergeDown ( iCurrentLayer );								
						}
						break;
					
					case SDLK_p:
						if ( bPanelsHidden == true )
						{
							bPanelsHidden = false;								
							updateScreen ( );														
						}
						else
						{
							bPanelsHidden = true;
							drawLayer ( );
						}
						break;		
					case SDLK_x:
						ptrTool->sendQuery ( "flipx" );
						break;
					case SDLK_y:
						ptrTool->sendQuery ( "flipy" );
						break;
					case SDLK_z:
						ptrTool->sendQuery ( "rotate" );
						break;
					case SDLK_k:
						if ( eEvent.key.keysym.mod == KMOD_RSHIFT || eEvent.key.keysym.mod == KMOD_LSHIFT )
						{
							oActiveCanvas->clear ( );
							drawLayer ( );
						}
						break;
					case SDLK_j:
						if ( bUsingSwap )
						{
							bUsingSwap = false;
							oActiveCanvas = &mCanvas[ iCurrentLayer ];
						}
						else
						{
							bUsingSwap = true;
							oActiveCanvas = &mCanvas[ iLayerNum - 2 ];								
						}																						
						drawLayer ( );
						break;
					case SDLK_d:											
						changeTool ( "pen" );
						break;
					case SDLK_f:													
						changeTool ( "fill" );
						break;
					case SDLK_l:
						changeTool ( "line" );
						break;
					case SDLK_c:
						changeTool ( "colorpicker" );							
						break;
					case SDLK_r:													
						changeTool ( "rectangle" );
						break;
					case SDLK_e:													
					
						/* Export here */							
						if ( eEvent.key.keysym.mod == KMOD_LCTRL || eEvent.key.keysym.mod == KMOD_RCTRL )							
						{												
							if ( sFileName.size ( ) )
							{
							
								setCursor ( mouseBusy );
																															
								for ( int b = 0; b < mCanvas[ 1 ].getFrames ( ); b++ )
								{
									if ( b == 0 )
									{
										// Rewind all layers
										for ( int a = 1; a < iLayerNum - 2; a++ )
										{
											mCanvas[ a ].rewind ( );
										}
									}
									else
									{
										// Rewind all layers
										for ( int a = 1; a < iLayerNum - 2; a++ )
										{
											mCanvas[ a ].nextFrame ( );
										}
									}																				
									
									mCanvas[ iLayerNum - 1 ].empty ( );
									
									/* Create an empty export surface in the image format */
									SDL_Surface* exportSurface = SDL_CreateRGBSurface ( 
										SDL_SWSURFACE, canvasWidth, canvasHeight, 32,										
										RMASK, GMASK, BMASK, AMASK 
									);																																												
									SDL_Rect rect; rect.x = 0; rect.y = 0; rect.w = canvasWidth; rect.h = canvasHeight;
																		
									/*
										Mix together all the pixels and blit them to the export surface:
									*/
									if ( iLayerNum - 2 > 1 )
									{
										for ( int ax = 0; ax < canvasWidth; ax++ )
										{
											for ( int ay = 0; ay < canvasHeight; ay++ )
											{
												cImageColor tCol; tCol.a = 0; tCol.r = 0; tCol.g = 0; tCol.b = 0;
																							
												for ( int a = 1; a < iLayerNum - 2; a++ )
												{																					
													mCanvas[ a ].addPixel ( ax, ay, &tCol );
												}											
												
												mCanvas[ 0 ].putPixel ( ax, ay, &tCol, exportSurface );
											}
										}
									}																		
									
									cFileIO file;
									stringstream tempFilename;
									
									// Include everything but file extension:
									for ( int a = 0; a < ( int )sFileName.size ( ) & sFileName.substr ( a, 1 ) != "."; a++ )
										tempFilename << sFileName.substr ( a, 1 );
										
									if ( iFrames > 1 )
										tempFilename << "_" << b;
									tempFilename << ".png";
									file.setFilename ( tempFilename.str ( ) );
									file.exportFile ( exportSurface, "PNG" );
									SDL_FreeSurface ( exportSurface );																											
								}
							}
							resetCursor ( );
							drawLayer ( );								
						}
						/*
							Export vertical!
						*/
						else if 
						(
							eEvent.key.keysym.mod == KMOD_LALT
						)					
						{
							if ( sFileName.size ( ) )
							{
							
								setCursor ( mouseBusy );
																																								
																																								
								/* Create an empty export surface in the image format */
								SDL_Surface* exportSurface = SDL_CreateRGBSurface ( 
									SDL_SWSURFACE, canvasWidth, canvasHeight * iFrames, 32,										
									RMASK, GMASK, BMASK, AMASK 
								);																																												
								SDL_Rect rect; rect.x = 0; rect.y = 0; rect.w = canvasWidth; rect.h = ( canvasHeight * iFrames );
								
								for ( int b = 0; b < mCanvas[ 1 ].getFrames ( ); b++ )
								{
									if ( b == 0 )
									{
										// Rewind all layers
										for ( int a = 1; a < iLayerNum - 2; a++ )
										{
											mCanvas[ a ].rewind ( );
										}
									}
									else
									{
										// Rewind all layers
										for ( int a = 1; a < iLayerNum - 2; a++ )
										{
											mCanvas[ a ].nextFrame ( );
										}
									}																				
									
									mCanvas[ iLayerNum - 1 ].empty ( );																				
																		
									/*
										Mix together all the pixels and blit them to the vertical 
										export surface:
									*/
									if ( iLayerNum - 2 > 1 )
									{
										for ( int ax = 0; ax < canvasWidth; ax++ )
										{
											for ( int ay = 0; ay < canvasHeight; ay++ )
											{
												cImageColor tCol; tCol.a = 0; tCol.r = 0; tCol.g = 0; tCol.b = 0;
																							
												for ( int a = 1; a < iLayerNum - 2; a++ )
												{																					
													mCanvas[ a ].addPixel ( ax, ay, &tCol );
												}											
												
												mCanvas[ 0 ].putPixel ( ax, ay + ( b * canvasHeight ), &tCol, exportSurface );
											}
										}
									}
								}
								
								cFileIO file;
								stringstream tempFilename;
								
								// Include everything but file extension:
								for ( int a = 0; a < ( int )sFileName.size ( ) & sFileName.substr ( a, 1 ) != "."; a++ )
									tempFilename << sFileName.substr ( a, 1 );																												
								
								tempFilename << ".png";
								
								file.setFilename ( tempFilename.str ( ) );
								file.exportFile ( exportSurface, "PNG" );
								
								SDL_FreeSurface ( exportSurface );	
							}
							resetCursor ( );
							drawLayer ( );	
						}
						else
						{
							changeTool ( "circle" );
						}
						break;						
					case SDLK_b:	
						
						if ( ptrTool->getName ( ) == "clipbrush" )												
						{
							ptrTool->sendQuery ( "clearBrushBuffer" );
							drawLayer ( );
						}
						else 
						{
							changeTool ( "clipbrush" );
						}
						
						break;							
																
					case SDLK_F1:											
						oBrush.setDrawMode ( "normal" );
						break;
					case SDLK_F5:
						oBrush.setDrawMode ( "lighten" );
						break;
					case SDLK_F6:							
						oBrush.setDrawMode ( "darken" );	
						break;						
					case SDLK_F7:							
						oBrush.setDrawMode ( "smudge" );		
						break;
					case SDLK_F8:							
						oBrush.setDrawMode ( "blur" );	
						break;
					case SDLK_F9:							
						oBrush.setDrawMode ( "erase" );
						break;
					case SDLK_F11:							
						oBrush.sendQuery ( "toggleantialias" );
						break;
					case SDLK_F12:							
						oBrush.sendQuery ( "togglefeather" );
						break;
					case SDLK_g:	
					
						if ( bUsingGrid )
						{
							bUsingGrid = false;
						}
						else
						{
							bUsingGrid = true;
						}
						break;
						
					case SDLK_HOME:	
						if ( iCurrentLayer + 1 < iLayerNum - 2 )
						{								
							iCurrentLayer ++;
							oLayers.setLayerNum ( iCurrentLayer - 1 );
							drawLayer ( );
						}
						oActiveCanvas = &mCanvas[ iCurrentLayer ];
						break;
					case SDLK_END:														
						if ( iCurrentLayer - 1 > 1 )
						{
							iCurrentLayer --;
							oLayers.setLayerNum ( iCurrentLayer - 1 );
							drawLayer ( );
						}
						oActiveCanvas = &mCanvas[ iCurrentLayer ];
						break;	
						
					case SDLK_s:
					
						if ( eEvent.key.keysym.mod == KMOD_LCTRL || eEvent.key.keysym.mod == KMOD_RCTRL )													
						{				
							
							// TODO: Can be optimized further
							
							cFileIO file;
							
							file.setWidth ( canvasWidth );
							
							file.setHeight ( canvasHeight );
							
							string tmpFileName = oFileDialogue.getFileName ( );																
							
							if ( tmpFileName.size ( ) )
							{										
								sFileName = tmpFileName; // <- store the filename globally ( for export )
								
								file.setFilename ( tmpFileName );
												
								file.setFrames ( mCanvas[ 1 ].getFrames ( ) );																
								
								file.setLayers ( iLayerNum - 3 );			
								
								int offset = 0;					
								
								for ( int a = 1; a < iLayerNum - 2; a++ )
								{
									mCanvas[ a ].rewind ( );									
									for ( int b = 0; b < mCanvas[ 1 ].getFrames ( ); b++ )
									{										
										file.copySurface ( mCanvas[ a ].getBuffer ( ), offset );
										mCanvas[ a ].nextFrame ( );
										offset++;
									}
								}																
								
								file.save ( );							
							}								
							drawLayer ( );								
						}												
						break;
						
					case SDLK_o:
						if ( eEvent.key.keysym.mod == KMOD_LCTRL || eEvent.key.keysym.mod == KMOD_RCTRL )
						{
							cFileIO file;																
							
							setCursor ( mouseArrow );
							
							string tempFileName;																
							
							if ( ( tempFileName = oFileDialogue.getFileName ( ) ) != "" )
							{									
								if ( tempFileName.size ( ) > 1 & tempFileName.substr ( tempFileName.size ( ) - 4, 4 ) == ".lun" )
								{
									sFileName = tempFileName;
																						
									file.setFilename ( sFileName );
									
									resetCursor ( );
									
									if ( file.getFilename ( ) )
									{																																		
										file.load ( );																			
																														
										mCanvas.clear ( );								
										
										iLayerNum = file.getLayers ( ) + 3;
										
										oLayers.setLayers ( iLayerNum - 3 );									
										
										oLayers.setScreenHeight ( iWindowHeight );																									
										
										setframes ( file.getFrames ( ) );																				
										
										setsize ( file.getWidth ( ), file.getHeight ( ) );
										
										int offset = 0;
										
										for ( int l = 1; l < iLayerNum - 2; l++ )
										{
											mCanvas[ l ].rewind ( );
											
											for ( int f = 0; f < iFrames; f++ )
											{
												file.copyFrameBuffer ( offset, mCanvas[ l ].getBuffer ( ) );
												mCanvas[ l ].nextFrame ( );																											
												offset++;																														
											}																									
											mCanvas[ l ].rewind ( );									
										}								
										
										calculateSkew ( );
										
										drawLayer ( );
										
										iCurrentLayer = 1;
										oActiveCanvas = &mCanvas[ iCurrentLayer ];
									}
									else
									{
										drawLayer ( );
									}
								}
								else
								{
									drawLayer ( );
								}
							}
							else
							{
								drawLayer ( );
							}
							
						}
						else
						{
							// Change onion skins...
							if ( iOnionSkin == 0 )
							{
								iOnionSkin = 1;
							}
							else if ( iOnionSkin == 1 )
							{
								iOnionSkin = 2;
							}
							else
							{
								iOnionSkin = 0;		
							}
							drawLayer ( );
							break;	
						}
						break;						
													
					case SDLK_i:
						if ( eEvent.key.keysym.mod == KMOD_LCTRL || eEvent.key.keysym.mod == KMOD_RCTRL )							
						{								
							mCanvas[ iCurrentLayer ].empty ( );
							cFileIO file;					
							string tempString = oFileDialogue.getFileName ( );
							if ( tempString.size ( ) > 4 )
							{
								if ( tempString.substr ( tempString.size ( ) - 4, 4 ) == ".png" )
								{
									file.setFilename ( tempString );
									setCursor ( mouseBusy );
									file.importFile ( &mCanvas[ iCurrentLayer ] );										
									resetCursor ( );
								}
							}
							drawLayer ( );
						}														
						break;
					default:
						break;
						
				}
				break;
			/* This one shall do - nothing! */
			default: break;
		}	
		SDL_GetMouseState ( &iMouseCoordX, &iMouseCoordY );
		if ( !bPanelsHidden )
			oPowerSlider.isMouseOver ( iMouseCoordX, iMouseCoordY );
	}					
	
	double mcx = 0.0f; double mcy = 0.0f;
	
	// Without an active grid
	if ( !bUsingGrid )
	{
		// Coorinate minus added skew and plus scroll offset on canvas
		mcx =  ( ( double )iMouseCoordX - iSkewX ) / canvasZoom + canvasOffsetX;
		mcy =  ( ( double )iMouseCoordY - iSkewY ) / canvasZoom + canvasOffsetY;			
	}
	// With a grid
	else
	{
		// Coorinate minus added skew and plus scroll offset on canvas
		mcx = round( ( ( double )( iMouseCoordX + iSkewX ) / canvasZoom + canvasOffsetX ) / iGridSize ) * iGridSize;
		mcy = round( ( ( double )( iMouseCoordY + iSkewY ) / canvasZoom + canvasOffsetY ) / iGridSize ) * iGridSize;
		/*mcx =  ( int )( ( ( double )iMouseCoordX - iSkewX ) / canvasZoom / 8 ) * 8 + canvasOffsetX;
		mcy =  ( int )( ( ( double )iMouseCoordY - iSkewY ) / canvasZoom / 8 ) * 8 + canvasOffsetY;	*/		
	}		
	
	/* get the color from the tool and buffer it */
	ptrTool->setCurrentColor ( oColorBuffer);
	
	cCoords* ActionCoords; // Holds the return actions from the tool action :-D	
	
	/* Retrieve results from the action */
	ActionCoords = 
		ptrTool->Action 	( 	oActiveCanvas, 
										mCanvas[ iLayerNum - 1 ].getBuffer ( ), ( int )mcx, 
										( int )mcy 
									);
												
	if ( ActionCoords->Activity )
	{				
		// Did we get Special flag to update whole screen?
		if ( ActionCoords->x == -999 && !ActionCoords->y )
		{
			ActionCoords->x = 0;
			ActionCoords->y = 0;
			ActionCoords->w = oWindow->w;
			ActionCoords->w = oWindow->h;
			drawLayer ( );
		}
		// No? Then do like normal
		else
		{				
			updateCoords ( 
				ActionCoords->x, ActionCoords->y, 
				ActionCoords->x + ActionCoords->w, ActionCoords->y + ActionCoords->h 
			);
		}
																				
		if ( canvasZoom > 1 )
		{				
			ActionCoords->x = ( ActionCoords->x - canvasOffsetX ) * canvasZoom;
			ActionCoords->y = ( ActionCoords->y - canvasOffsetY ) * canvasZoom;
			ActionCoords->w = ActionCoords->w * canvasZoom;
			ActionCoords->h = ActionCoords->h * canvasZoom;
		}
		else
		{
			ActionCoords->x -= canvasOffsetX;
			ActionCoords->y -= canvasOffsetY;
		}
					
		/* Not out of screen */
		if ( ActionCoords->x < 0 )
			ActionCoords->x = 0;
		if ( ActionCoords->y < 0 )
			ActionCoords->y = 0;
		if ( ActionCoords->x + ActionCoords->w > oWindow->w )
			ActionCoords->w = oWindow->w - ActionCoords->x;
		if ( ActionCoords->y + ActionCoords->h > oWindow->h )
			ActionCoords->h = oWindow->h - ActionCoords->y;	
															
		// Only update area around canvas
		if ( ActionCoords->x < 0 )
			ActionCoords->x = 0;
		if ( ActionCoords->y < 0 )
			ActionCoords->y = 0;
		if ( ActionCoords->x + ActionCoords->w > ( canvasWidth * canvasZoom ) )
			ActionCoords->w = ( canvasWidth * canvasZoom ) - ActionCoords->x;
		if ( ActionCoords->y + ActionCoords->h > ( canvasHeight * canvasZoom ) )
			ActionCoords->h = ( canvasHeight * canvasZoom ) - ActionCoords->y;
		
		// Add skew						
		ActionCoords->x += iSkewX;
		ActionCoords->y += iSkewY;
			
		if ( ActionCoords->w >= 1 & ActionCoords->h >= 1 )
			SDL_UpdateRect ( oWindow, ActionCoords->x, ActionCoords->y, ActionCoords->w, ActionCoords->h );
			if ( ptrTool->getName ( ) == "colorpicker" )
		{
			oColorBuffer= ptrTool->getCurrentColor ( );
			oBrush.setColorPointer ( &oColorBuffer );
		}
	}
	
	/* Store these coordinates as previous coordinates */
	iPrevX = ( int )mcx;
	iPrevY = ( int )mcy;									
	
	/* How to show the mouse.. */
	if ( iMouseCoordX > oWindow->w - 17 & bPanelsHidden == false )
	{
		if ( sUsingCursor != "normal" )
		{
			setCursor ( mouseArrow );
			sUsingCursor = "normal";
		}
	}
	else if ( iMouseCoordX < 16 & bPanelsHidden == false )
	{	
		if ( sUsingCursor != "normal" )
		{
			setCursor ( mouseArrow );
			sUsingCursor = "normal";
		}	
	}
	else
	{
		if ( sUsingCursor != "crosshair" )
		{
			setCursor ( mouseCrosshair );
			sUsingCursor = "crosshair";
		}
	}
}

void cToolkit::changeTool ( string toolname )
{
	/* Empty realtime layer */		
	drawLayer ( );
	
	/* Set new tool */
	if ( toolname == "pen" )
		ptrTool =& toolbrush;
	else if ( toolname == "fill" )		
		ptrTool =& toolfill;
	else if ( toolname == "colorpicker" )
		ptrTool =& toolcolorpicker;
	else if ( toolname == "rectangle" )
	{
		if ( ptrTool->getName ( ) == "rectangle" )
		{
			ptrTool->sendQuery ( "toggleFill" );
		}
		ptrTool =& toolrectangle;
	}
	else if ( toolname == "clipbrush" )
	{
		ptrTool =& toolclipbrush;			
	}
	else if ( toolname == "circle" )
	{
		if ( ptrTool->getName ( ) == "circle" )
		{
			ptrTool->sendQuery ( "toggleFill" );
		}
		ptrTool =& toolcircle;
	}
	else if ( toolname == "line" )
		ptrTool =& toolline;
		
	ptrTool->setBrush ( &oBrush );		
}

	
void cToolkit::changeZoom ( int number, int oX = 0, int oY = 0 )
{	
	/* ox and oy will be used for mouse coord focus */				
	oX = 0; oY = 0; 		
	
	int newZoom = ( canvasZoom + number );
	
	if ( newZoom > 0 & newZoom < 20 )
	{			
		/* Check offset */
		if ( newZoom == 1 )
		{
			/* All zoomed out doesn't need any offset */
			canvasOffsetX = 0;
			canvasOffsetY = 0;
		}
		else
		{			
			/*
				Only calculate new offset if we are zoomed in
				more than screen width can show:
			*/
			if ( canvasLargerThanScreen_W ( ) )
			{
				if ( number > 0 )
				{
					canvasOffsetX += ( int )( ( double )( iWindowWidth / newZoom ) / 8 );
				}
				else
				{
					canvasOffsetX -= ( int )( ( double )( iWindowWidth / newZoom ) / 8 );				
				}
			}
			/*
				Only calculate new offset if we are zoomed in
				more than screen height can show:
			*/
			if ( canvasLargerThanScreen_H ( ) )
			{
				if ( number > 0 )
				{
					canvasOffsetY += ( int )( ( double )( iWindowHeight / newZoom ) / 8 );
				}
				else
				{
					canvasOffsetY -= ( int )( ( double )( iWindowHeight / newZoom ) / 8 );
				}
			}
		}				
		
		/* Apply zoom! */			
		canvasZoom = newZoom;					
		
		/* Check bounds */
		if ( canvasOffsetX < 0 )
			canvasOffsetX = 0;
		if ( canvasOffsetY < 0 )
			canvasOffsetY = 0;
	}				
	calculateSkew ( );
}

void cToolkit::moveZoom ( int direction )
{	
	int step = ( int )( ( double )40 / canvasZoom );
	
	if ( 	canvasLargerThanScreen_W ( ) )
	{								
		switch ( direction )
		{
			// 1 = left
			case 1:
				if ( canvasOffsetX - step >= 0 )
					canvasOffsetX -= step;
				else
					canvasOffsetX = 0;
				break;
			// 2 = right			
			case 2:
				if ( canvasOffsetX + step <= canvasWidth - ( int )( ( double )oWindow->w / canvasZoom ) )
					canvasOffsetX += step;
				else
					canvasOffsetX = ( canvasWidth ) - ( int )( ( double )oWindow->w / canvasZoom );
				break;
			default:
				break;
		}
	}
	if ( canvasLargerThanScreen_H ( ) )
	{
		switch ( direction )
		{
			// 3 = up
			case 3:
				if ( canvasOffsetY - step >= 0 )
					canvasOffsetY -= step;
				else
					canvasOffsetY = 0;
				break;
			// 4 = down
			case 4:				
				if ( canvasOffsetY + step <= canvasHeight  - ( int )( ( double )oWindow->h / canvasZoom ) )
					canvasOffsetY += step;
				else
					canvasOffsetY = canvasHeight - ( int )( ( double )oWindow->h / canvasZoom );
				break;
			default: 
				break;
		}
	}
	calculateSkew ( );
}
	
void cToolkit::updateScreen ( bool update )
{		
	/* For the panels */
	if ( !bPanelsHidden )
	{
		oPalette.redraw ( oWindow );
		oLayers.redraw ( oWindow );			
		oTools.redraw ( oWindow );
		
		// Draw current frame number			
		stringstream tempFrameNum; 
		if ( mCanvas[ 1 ].getCurrentFrame ( ) < 9 )
			tempFrameNum << "0";
		tempFrameNum << ( mCanvas[ 1 ].getCurrentFrame ( ) + 1 );
		
		putSmallText ( 3, 179, tempFrameNum.str ( ) );
		
		if ( bUsingSwap )
		{
			putSmallText ( 4, 147, "S" );
		}
		else
		{
			putSmallText ( 4, 147, "M" );
		}
		
		if ( update )
		{
			SDL_UpdateRect ( oWindow, oWindow->w - 21, 0, 20, oWindow->h );
			SDL_UpdateRect ( oWindow, 0, 0, 20, oWindow->h );
		}
		
		/* 
			Update the tool power slider:#include <iostream>
		*/
		oPowerSlider.refresh ( );
	}		
}

void cToolkit::setsize ( int width, int height )
{		
	canvasWidth = width;
	canvasHeight = height;
	
	oTempBuffer = SDL_CreateRGBSurface ( SDL_SWSURFACE, width, height, 32, RMASK, GMASK, BMASK, AMASK );		

	/* Set height to layers panel */
	oLayers.setScreenHeight ( iWindowHeight );
	
	/* Fill the bottom layer */		
	mCanvas[ 0 ].resize ( width, height );		
	
	SDL_Rect rect; rect.x = 0; rect.y = 0; rect.w = width; rect.h = height;
	
	SDL_Surface* buffer = mCanvas[ 0 ].getBuffer( );
	
	Uint32 col = SDL_MapRGBA ( buffer->format, 200, 200, 200, 255 );			
	Uint32 ch1 = SDL_MapRGBA ( buffer->format, 100, 100, 100, 255 );
	Uint32 ch2 = SDL_MapRGBA ( buffer->format, 120, 120, 120, 255 );		
	
	SDL_Rect boxrect; 
	boxrect.x = 0; boxrect.y = 0; 
	boxrect.w = 8; 
	boxrect.h = 8;		
	
	SDL_FillRect( buffer, &rect, col );	
	SDL_FillRect( oTempBuffer, &rect, col );	
	
	mCanvas[ 0 ].setTempBuffer ( oTempBuffer );		
	
	int a, b;
	
	int ySwitcher = 0;				
	int switcher = 0;
			
	for ( b = 0; b < height; b += boxrect.h )
	{		
		if ( ySwitcher ==	1 )
		{
			ySwitcher = 0; 
			switcher = 0;
		}
		else 
		{
			ySwitcher = 1;
			switcher = 1;
		}
									
		for ( a = 0; a < width; a += boxrect.w )
		{	
			rect.x = a; rect.y = b;
			
			if ( switcher == 0 )
			{									
				switcher = 1;
				SDL_FillRect ( buffer, &rect, ch2 );
			}
			else
			{					
				switcher = 0;
				SDL_FillRect ( buffer, &rect, ch1 );
			}								
		}			
	}
					
	for ( a = 1; a < iLayerNum; a++ )
	{
		mCanvas[ a ].resize ( width, height );
		mCanvas[ a ].setTempBuffer ( oTempBuffer );
	}
}

void cToolkit::loadImage ( string filename )
{
	cFileIO file;
	file.setFilename ( filename );
	if ( file.importFile ( &mCanvas[ iCurrentLayer ] ) )
	{
		sFileName = filename;
		drawLayer ( );
	}
	else
		sFileName = filename;
}

// TODO: Check for memory capability
void cToolkit::setframes ( int num )
{
	for ( int a = 1; a < iLayerNum - 2; a++ )
		mCanvas[ a ].setFrames ( num );
	iFrames = num;
}	

void cToolkit::calculateSkew ( )
{		
	// Canvas skew if image smaller than screen
	iSkewX = ( int )( ( oWindow->w - ( canvasWidth * canvasZoom ) ) / 2 );
	iSkewY = ( int )( ( oWindow->h - ( canvasHeight * canvasZoom ) ) / 2 );
	if ( iSkewX < 0 ) iSkewX = 0;
	if ( iSkewY < 0 ) iSkewY = 0;	
}

bool cToolkit::canvasLargerThanScreen ( )
{
	if ( canvasWidth * canvasZoom > oWindow->w or canvasHeight * canvasZoom > oWindow->h )		
		return true;
	else
		return false;
}
bool cToolkit::canvasLargerThanScreen_W ( )
{
	if ( canvasWidth * canvasZoom > oWindow->w )		
		return true;
	else
		return false;
}
bool cToolkit::canvasLargerThanScreen_H ( )
{
	if ( canvasHeight * canvasZoom > oWindow->h )		
		return true;
	else
		return false;
}

void cToolkit::resetCursor ( )
{
	if ( sUsingCursor == "normal" )
	{
		setCursor ( mouseArrow );
	}
	else if ( sUsingCursor == "crosshair" )
	{
		setCursor ( mouseCrosshair );
	}
}

void cToolkit::putSmallText ( int x, int y, string text )
{
	SDL_Color col1 = { 255, 255, 255, 255 };
	SDL_Color col2 = { 0, 0, 0, 0 };
	
	SDL_Surface* textSurf = TTF_RenderText_Shaded ( oSmallFont, text.c_str ( ), col1, col2 );
	SDL_Rect rect; rect.x = x; rect.y = y; rect.w = textSurf->w; rect.h = textSurf->h;
	SDL_BlitSurface ( textSurf, NULL, oWindow, &rect );
	
	SDL_FreeSurface ( textSurf );
}

void cToolkit::mergeDown ( int layerNum )
{
	setCursor ( mouseBusy );
	if ( layerNum > 1 )
	{
		SDL_Rect rect = { 0, 0, mCanvas[ 0 ].getBuffer ( )->w, mCanvas[ 0 ].getBuffer ( )->h };
		
		for ( int ax = 0; ax < rect.w; ax++ )
		{
			for ( int bx = 0; bx < rect.h; bx++ )
			{
				cImageColor color = mCanvas[ layerNum - 1 ].getPixel ( ax, bx );
				mCanvas[ layerNum ].addPixel ( ax, bx, &color );
				mCanvas[ layerNum - 1 ].putPixel ( ax, bx, color  );
			}
		}
			
		mCanvas[ layerNum ].empty ( );
		drawLayer ( );
	}		
	resetCursor ( );
}

