#ifndef _CTOOLKIT_H_
#define _CTOOLKIT_H_

#include <iostream>
#include <string>
#include <sstream>
#include <cmath>

#include <SDL/SDL.h>		
#include <SDL/SDL_ttf.h>

#include "../tools/cTool.h"
#include "../tools/cToolBrush.h"
#include "../tools/cToolFill.h"
#include "../tools/cToolColorpicker.h"
#include "../tools/cToolClipbrush.h"
#include "../tools/cToolRectangle.h"
#include "../tools/cToolCircle.h"
#include "../tools/cToolLine.h"

#include "../main/cImage.h"
#include "../main/cBrush.h"
#include "../main/cFileIO.h"

#include "../panels/cPanelPalette.h"
#include "../panels/cPanelLayers.h"
#include "../panels/cPanelTools.h"

#include "../gui/cFileDialogue.h"
#include "../gui/cGadgetImageButton.h"		
#include "../gui/cGadgetSlider.h"

#include "../globalcode/code_mouse_pointers.cpp"	

using namespace std;

class cToolkit
{
	private:							
		/* For making surfaces */
		Uint32 RMASK, GMASK, BMASK, AMASK;
		
		/* The window we're gonna output to */
		
		SDL_Surface* oWindow;						
		
		/* The font for use with small text */
		
		TTF_Font* oSmallFont;
		
		/* The file dialogue for choosing file to load and save */
		
		cFileDialogue oFileDialogue;
		
		/* Frames we're working on */
		int iFrames;
		
		/* Current grid size */
		int iGridSize;
		
		// Current cursor
		string sUsingCursor;
		
		/* Currentlayer */
		int iCurrentLayer;
		
		int iWindowWidth;
		int iWindowHeight;
		
		int iOnionSkin;
		
		int iLayerNum;
		
		// If we're using grid or swap screen 
		bool bUsingGrid, bUsingSwap;	
		
		//Canvas skew (if image is smaller than window )
		int iSkewX;
		int iSkewY;									
		
		/* The canvas we're currently drawing on ( four ones )*/
		map <int, cImage > mCanvas;
		
		/* A temporary buffer that can be written to */
		SDL_Surface* oTempBuffer;
		
		/* A main buffer used to project on window */
		SDL_Surface* oMainBuffer;			
		
		/* A ptr to the current layer canvas */
		cImage* oActiveCanvas;
		
		/* The pointer to the current tool we're using */
		cTool* ptrTool;						
		
		/* The input event listener */
		SDL_Event eEvent;						
		
		/* If we are fullscreen or not */
		bool bFullscreen;		
		
		int iPrevX, iPrevY; // Prev coordinates, look in .cpp
		int iMouseCoordX, iMouseCoordY; // Current coordinates, look in .cpp
		int iCurrentFrame; // Current frame number			
		int iDisplayBPP; // The bit depth of the display
		Uint8* displayPTRpixels;
		int displayPitch;
		
		/* Library of tools to link to cTool tool */
		cToolBrush toolbrush;
		cToolFill toolfill;
		cToolColorpicker toolcolorpicker;		
		cToolClipbrush toolclipbrush;
		cToolRectangle toolrectangle;
		cToolCircle toolcircle;
		cToolLine toolline;
		
		/* If main panels are hidden or not */
		bool bPanelsHidden;
		
		/* Panels */
		cPanelPalette oPalette;
		cPanelLayers oLayers;
		cPanelTools oTools;
		
		/* Colorbuffer */
		cImageColor oColorBuffer;
		
		/* filename */
		string sFileName;
								
		int canvasOffsetX, canvasOffsetY, canvasZoom, canvasWidth, canvasHeight;						
								
		cGadgetSlider oPowerSlider;
								
	public:
		
		cToolkit ( int argLayers );
		~cToolkit ( );						
		
		cBrush oBrush;
		
		void setOutput ( SDL_Surface* window, int width, int height );
		
		void Action ( );
		
		void changeTool ( string name );
		
		inline void drawLayer ( );			
		inline void updateCoords ( int x1, int y1, int x2, int y2 );			
		
		void changeZoom ( int number, int oX, int oY );
		void moveZoom ( int direction );
		
		void updateScreen ( bool update = true );
		
		void setsize ( int width, int height );
		void setframes ( int num );
		
		void loadImage ( string filename );
		
		inline void putPixel ( int x, int y, cImageColor color );
		inline void putPixel ( int x, int y, Uint32 color );
		inline void putPixel ( int x, int y, Uint32 color, SDL_Surface* surface );
		
		void calculateSkew ( );
		bool canvasLargerThanScreen ( );
		bool canvasLargerThanScreen_W ( );
		bool canvasLargerThanScreen_H ( );
		
		bool quitDirective; // If we're quitting or not...
		
		void resetCursor ( );
		
		void putSmallText ( int x, int y, string text );
		
		void mergeDown ( int layerNum );
};

/* Redraw entire screen */
inline void cToolkit::drawLayer ( )
{			
	setCursor ( mouseBusy );		

	/* Empty screen and draw border if the canvas does not cover the whole view area */			
	mCanvas[ iLayerNum - 1 ].clear ( );		
	
	if ( 
			canvasWidth - canvasOffsetX * canvasZoom < oWindow->w || 
			canvasHeight - canvasOffsetY * canvasZoom < oWindow->h 
	)
	{
		SDL_Rect flushRect; flushRect.x = 0; flushRect.y = 0; flushRect.w = oWindow->w; flushRect.h = oWindow->h;
		
		Uint32 flushCol = SDL_MapRGB ( oWindow->format, 128, 128, 128 );						
		
		SDL_FillRect ( oWindow, &flushRect, flushCol );
		
		if ( iSkewX > 0 & iSkewY > 0 )
		{			
			flushCol = SDL_MapRGB ( oWindow->format, 255, 255, 255 );
			flushRect.x = iSkewX - 1; flushRect.y = iSkewY - 1;
			flushRect.w = ( canvasWidth * canvasZoom ) + 2;
			flushRect.h = ( canvasHeight * canvasZoom ) + 2;
			SDL_FillRect ( oWindow, &flushRect, flushCol );
		}
	}						
	// Done filling border and emptying screen												
	
	// Determine how far we can fill when zooming
	int maxwidth = canvasWidth;
	int maxheight = canvasHeight;
	
	if ( maxwidth * canvasZoom > oWindow->w )
		maxwidth = oWindow->w / canvasZoom;
	if ( maxheight * canvasZoom > oWindow->h )
		maxheight = oWindow->h / canvasZoom;
	// done determening
	
	/* If on the swap screen */
	if ( bUsingSwap )
	{		
		// New routine
		// TODO: Fix it with /zoom thing
		if ( canvasZoom > 1 )
		{				
			// The area to grab from
			SDL_Rect rect, dest; 
			rect.x = canvasOffsetX; 
			rect.y = canvasOffsetY;
			rect.w = maxwidth;
			rect.h = maxheight;						
			
			dest.x = 0; 
			dest.y = 0;
			dest.w = maxwidth;
			dest.h = maxheight;																	
							
			// Blit over surface				
			SDL_BlitSurface ( mCanvas[ 0 ].getBuffer ( ), &rect, oTempBuffer, &dest );								
			SDL_BlitSurface ( mCanvas[ iLayerNum - 2 ].getBuffer ( ), &rect, oTempBuffer, &dest );																															
																											
			for ( int x = 0; x < maxwidth; x ++ )
			{
				for ( int y = 0; y < maxheight; y ++ )
				{																							
					cImageColor color = mCanvas[ 0 ].getPixel ( x, y, oTempBuffer );
					
					int xx = ( x * canvasZoom ) + iSkewX;
					int yy = ( y * canvasZoom ) + iSkewY;
					int xw = xx + canvasZoom;
					int yh = yy + canvasZoom;
																							
					for ( int a = xx; a < xw; a++ )
					{
						for ( int b = yy; b < yh; b++ )
						{
							putPixel ( a, b, color );
						}
					}																								
				}					
			}				
		}
		else
		{				
			SDL_Rect rect; 
			rect.x = canvasOffsetX; rect.y = canvasOffsetY; 				
			rect.w = canvasWidth; 
			rect.h = canvasHeight;
			SDL_Rect dest;
			dest.x = iSkewX; dest.y = iSkewY; dest.w = rect.w; dest.h = rect.h;
							
			// Blit over surface
			SDL_BlitSurface ( mCanvas[ 0 ].getBuffer ( ), &rect, oWindow, &dest );
			SDL_BlitSurface ( mCanvas[ iLayerNum - 2 ].getBuffer ( ), &rect, oWindow, &dest );								
		}
	}
	/* If on the main drawing screen */
	else
	{							
		// Redraw the entire screen with x zoom	
		if ( canvasZoom > 1 )
		{	
			// The area to grab from
			SDL_Rect rect, dest; 
			rect.x = canvasOffsetX; 
			rect.y = canvasOffsetY;
			rect.w = maxwidth;
			rect.h = maxheight;						
			
			dest.x = 0; 
			dest.y = 0;
			dest.w = maxwidth;
			dest.h = maxheight;																	
							
			// Blit over surface				
			SDL_BlitSurface ( mCanvas[ 0 ].getBuffer ( ), &rect, oTempBuffer, &dest );
			
			for ( int layer = 0; layer < iLayerNum - 2; layer++ )
			{
				if ( layer == iCurrentLayer & iOnionSkin > 0 )
				{						
					// Add a temporary surface to blit through
					SDL_Surface* tempSurface = SDL_CreateRGBSurface ( SDL_SWSURFACE, maxwidth, maxheight, 32, RMASK, GMASK, BMASK, AMASK );						
					for ( int x = 0; x < maxwidth; x ++ )
					{																				
						for ( int y = 0; y < maxheight; y ++ )
						{		
							cImageColor colA;
								
							if ( iOnionSkin == 1 )
							{
								colA = 
									mCanvas[ 0 ].getPixel ( 
										x + canvasOffsetX, y + canvasOffsetY, mCanvas[ layer ].getPrevFrameBuffer ( ) 
								);
							}
							else
							{ 
								colA = 
									mCanvas[ 0 ].getPixel ( 
										x + canvasOffsetX, y + canvasOffsetY, mCanvas[ layer ].getNextFrameBuffer ( ) 
								);
							}
							
							putPixel ( 
								x, y, SDL_MapRGBA 
									( tempSurface->format, colA.r, colA.g, colA.b, ( int )( ( double ) colA.a / 3 ) ), 
								tempSurface 
							);									
						}
					}												
					SDL_BlitSurface ( tempSurface, NULL, oTempBuffer, &dest );
					// Done
				}															
				SDL_BlitSurface ( mCanvas[ layer ].getBuffer ( ), &rect, oTempBuffer, &dest );
			}																																
																											
			for ( int x = 0; x < maxwidth; x ++ )
			{
				for ( int y = 0; y < maxheight; y ++ )
				{																							
					cImageColor color = mCanvas[ 0 ].getPixel ( x, y, oTempBuffer );
					
					int xx = ( x * canvasZoom ) + iSkewX;
					int yy = ( y * canvasZoom ) + iSkewY;
					int xw = xx + canvasZoom;
					int yh = yy + canvasZoom;
																							
					for ( int a = xx; a < xw; a++ )
					{
						for ( int b = yy; b < yh; b++ )
						{
							putPixel ( a, b, color );
						}
					}																								
				}					
			}					
		}
		// Redraw the entire screen when zoom = 1
		else
		{																			
		
			int width = canvasWidth;
			int height = canvasHeight;
											
			SDL_Rect rect; 
			
			rect.x = canvasOffsetX; 
			rect.y = canvasOffsetY; 				
			rect.w = width;
			rect.h = height;
			
			SDL_Rect dest;
			
			dest.x = iSkewX; dest.y = iSkewY; dest.w = rect.w; dest.h = rect.h;																
			
			for ( int layer = 0; layer < iLayerNum - 2; layer ++ )
			{
				// use onion skin
				if ( layer == iCurrentLayer & iOnionSkin > 0 )
				{
					// Make surface clear to draw on
					Uint32 col = SDL_MapRGBA ( oTempBuffer->format, 100, 100, 100, 0 );
					SDL_FillRect ( oTempBuffer, &dest, col );
					
					for ( int x = 0; x < width; x ++ )
					{
						for ( int y = 0; y < height; y ++ )
						{		
							cImageColor pixel;
							
							if ( iOnionSkin == 1 )
							{						
								pixel = mCanvas[ layer ].getPixel ( x, y, oActiveCanvas->getPrevFrameBuffer( ) );
							}
							else
							{
								pixel = mCanvas[ layer ].getPixel ( x, y, oActiveCanvas->getNextFrameBuffer( ) );
							}
							
							if ( pixel.a > 0 )								
							{
								col = SDL_MapRGBA ( oTempBuffer->format, pixel.r, pixel.g, pixel.b, ( int )( ( double )pixel.a / 3 ) );									
								putPixel ( 
									x - canvasOffsetX, 
									y - canvasOffsetY,
									col, oTempBuffer																	
								);
							}
						}
					}											
					SDL_BlitSurface ( oTempBuffer, &rect, oWindow, &dest );						
					// Make the surface clear to draw on again.....
					SDL_FillRect ( oTempBuffer, &dest, col );
				}	
													
				SDL_BlitSurface ( mCanvas[ layer ].getBuffer ( ), &rect, oWindow, &dest );					
			}							
		}
	}														
	updateScreen ( false );						
	SDL_Flip ( oWindow );		
			
	resetCursor ( );		
}

/* Update screen with changes on coodinates */
inline void cToolkit::updateCoords ( int x1, int y1, int x2, int y2 )
{																
	if ( x1 < 0 )
		x1 = 0;
	if ( y1 < 0 )
		y1 = 0;
	
	if ( bUsingSwap )
	{
		// Fast routine
		if ( canvasZoom > 1 )
		{
			int x, y = 0; cImageColor col;				
			for ( x = x1; x < x2; x ++ )
			{
				for ( y = y1; y < y2; y ++ )
				{
					col.r = 0; col.g = 0; col.b = 0; col.a = 0;												
					mCanvas[ 0 ].addPixel ( x, y, &col );
					mCanvas[ iLayerNum - 2 ].addPixel ( x, y, &col );
					mCanvas[ iLayerNum - 1 ].addPixel ( x, y, &col );						
					SDL_Rect rect; 
					rect.x = ( x - canvasOffsetX ) * canvasZoom + iSkewX;  
					rect.y = ( y - canvasOffsetY ) * canvasZoom + iSkewY; 
					rect.h = canvasZoom; rect.w = canvasZoom;
					Uint32 colorW = SDL_MapRGBA ( oWindow->format, col.r, col.g, col.b, col.a );
					SDL_FillRect ( oWindow, &rect, colorW );
				}
			}
		}
		else
		{						
			Uint32 testCol = SDL_MapRGBA ( oTempBuffer->format, 0, 0, 0, 255 );
			SDL_Rect tempRect; tempRect.x = x1; tempRect.y = y1; tempRect.w = x2 - x1; tempRect.h = y2 - y1;				
			SDL_Rect destRect; destRect.x = 0; destRect.y = 0; destRect.w = x2 - x1; destRect.h = y2 - y1;
			SDL_FillRect ( oTempBuffer, &destRect, testCol );
			SDL_BlitSurface ( mCanvas[ 0 ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			SDL_BlitSurface ( mCanvas[ iLayerNum - 2 ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			SDL_BlitSurface ( mCanvas[ iLayerNum - 1 ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			
			for ( int x = x1; x < x2; x ++ )
			{
				for ( int y = y1; y < y2; y ++ )
				{
					cImageColor col = mCanvas[ 0 ].getPixel ( x - x1, y - y1, oTempBuffer );
					putPixel ( x + iSkewX, y + iSkewY, col );
				}
			}						
		}
	}
	else
	{	
		// Fast routine -- zooming in
		if ( canvasZoom > 1 )
		{
			Uint32 testCol = SDL_MapRGB ( oTempBuffer->format, 0, 0, 0 );
			SDL_Rect tempRect; tempRect.x = x1; tempRect.y = y1; tempRect.w = x2 - x1; tempRect.h = y2 - y1;				
			SDL_Rect destRect; destRect.x = 0; destRect.y = 0; destRect.w = x2 - x1; destRect.h = y2 - y1;
			SDL_FillRect ( oTempBuffer, &destRect, testCol );								
			
			for ( int layer = 0; layer < iLayerNum - 2; layer++ )
			{
				// onionskin
				if ( layer == iCurrentLayer & iOnionSkin > 0 )
				{
					for ( int x = x1; x < x2; x ++ )
					{																				
						for ( int y = y1; y < y2; y ++ )
						{	
							cImageColor colB = mCanvas[ 0 ].getPixel ( x - x1, y - y1, oTempBuffer );
							cImageColor colA;							
							
							if ( iOnionSkin == 1 )
							{
								colA = mCanvas[ 0 ].getPixel ( x, y, mCanvas[ layer ].getPrevFrameBuffer ( ) );									
							}
							else
							{
								colA = mCanvas[ 0 ].getPixel ( x, y, mCanvas[ layer ].getNextFrameBuffer ( ) );									
							}
							if ( colA.a > 0 )
							{												
								colA.a = ( int )( ( double )colA.a / 3 );
								mCanvas[ 0 ].combinePixels ( &colA, colB );
								mCanvas[ 0 ].putPixel ( x - x1, y - y1, &colA, oTempBuffer );
							}
						}
					}																						
					// Done												
				}
				SDL_BlitSurface ( mCanvas[ layer ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			}
			SDL_BlitSurface ( mCanvas[ iLayerNum - 1 ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			
							
			for ( int x = x1; x < x2; x ++ )
			{
				for ( int y = y1; y < y2; y ++ )
				{
					cImageColor col = mCanvas[ 0 ].getPixel ( x - x1, y - y1, oTempBuffer );
												
					int xx = ( ( x - canvasOffsetX ) * canvasZoom ) + iSkewX;
					int yy = ( ( y - canvasOffsetY ) * canvasZoom ) + iSkewY;
					int xw = xx + canvasZoom;
					int yh = yy + canvasZoom;
					
					Uint32 colU = SDL_MapRGBA ( oWindow->format, col.r, col.g, col.b, col.a );
					
					for ( int a = xx; a < xw; a++ )
					{
						for ( int b = yy; b < yh; b++ )
						{
							putPixel ( a, b, colU );
						}
					}
				}
			}
		}
		// NEW routine!
		else
		{		
			Uint32 testCol = SDL_MapRGBA ( oTempBuffer->format, 0, 0, 0, 255 );
			SDL_Rect tempRect; tempRect.x = x1; tempRect.y = y1; tempRect.w = x2 - x1; tempRect.h = y2 - y1;				
			SDL_Rect destRect; destRect.x = 0; destRect.y = 0; destRect.w = x2 - x1; destRect.h = y2 - y1;
			SDL_FillRect ( oTempBuffer, &destRect, testCol );
			for ( int layer = 0; layer < iLayerNum - 2; layer++ )
			{
				// onionskin
				if ( layer == iCurrentLayer & iOnionSkin > 0 )
				{
					for ( int x = x1; x < x2; x ++ )
					{																				
						for ( int y = y1; y < y2; y ++ )
						{	
							cImageColor colB = mCanvas[ 0 ].getPixel ( x - x1, y - y1, oTempBuffer );
							cImageColor colA;
							
							if ( iOnionSkin == 1 )
							{
								colA = mCanvas[ 0 ].getPixel ( x, y, mCanvas[ layer ].getPrevFrameBuffer ( ) );
							}
							else
							{
								colA = mCanvas[ 0 ].getPixel ( x, y, mCanvas[ layer ].getNextFrameBuffer ( ) );
							}
							
							if ( colA.a > 0 )
							{												
								colA.a = ( int )( ( double )colA.a / 3 );
								mCanvas[ 0 ].combinePixels ( &colA, colB );
								mCanvas[ 0 ].putPixel ( x - x1, y - y1, &colA, oTempBuffer );
							}
						}
					}																						
					// Done												
				}
				SDL_BlitSurface ( mCanvas[ layer ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			}
			SDL_BlitSurface ( mCanvas[ iLayerNum - 1 ].getBuffer ( ), &tempRect, oTempBuffer, &destRect );
			
			for ( int x = x1; x < x2; x ++ )
			{
				for ( int y = y1; y < y2; y ++ )
				{
					cImageColor col = mCanvas[ 0 ].getPixel ( x - x1, y - y1, oTempBuffer );
					putPixel ( x + iSkewX, y + iSkewY, col );
				}
			}																																	
		}
	}					
	updateScreen ( false );
}

inline void cToolkit::putPixel ( int x, int y, cImageColor color )
{				
	putPixel ( x, y, SDL_MapRGBA ( oWindow->format, color.r, color.g, color.b, color.a ) );				
}		

inline void cToolkit::putPixel ( int x, int y, Uint32 pixel )
{		
	if ( x >= 0 & x < iWindowWidth & y >= 0 & y < iWindowHeight )
	{																
		/* Here p is the address to the pixel we want to set */
		Uint8* p = displayPTRpixels + y * displayPitch + x * iDisplayBPP;	

		switch( iDisplayBPP ) 
		{
			case 1:
				*p = pixel;
				break;			
			case 2:
				*( Uint16* )p = pixel;
				break;			
			case 3:
				if( SDL_BYTEORDER == SDL_BIG_ENDIAN ) 
				{
					p[ 0 ] = ( pixel >> 16 ) & 0xff;
					p[ 1 ] = ( pixel >> 8 ) & 0xff;
					p[ 2 ] = pixel & 0xff;
				} 
				else 
				{
					p[ 0 ] = pixel & 0xff;
					p[ 1 ] = ( pixel >> 8 ) & 0xff;
					p[ 2 ] = ( pixel >> 16 ) & 0xff;
				}
				break;			
			case 4:
				*( Uint32* )p = pixel;
				break;
		}
	}		
}	
	
inline void cToolkit::putPixel ( int x, int y, Uint32 color, SDL_Surface* surface )
{		
	if ( x >= 0 & x < iWindowWidth & y >= 0 & y < iWindowHeight )
	{																
		/* Quickly place a pixel */
		*( ( Uint32* )surface->pixels + y * surface->pitch / 4 + x ) = color;
	}		
}	

#endif
