
	#include "cToolBrush.h"
	
	cToolBrush::cToolBrush ( )
	{
		fPI = 3.14159265358979323846264f;
		fRAD = fPI / 180.0f;
		sName = "pen";
		sMouseMode = "off";	
		iPrevX = -1;
		iPrevY = -1;
		bIsDrawing = false;
	}
	
	cToolBrush::~cToolBrush ( )
	{
		sName = "";
		sMouseMode = "";		
	}
	
	/*
		The "action" function is the brain of the brush tool.
		Everything central happens here - directives, usage of
		other functions etc... 
	*/
	cCoords* cToolBrush::Action ( cImage* canvas, SDL_Surface* window, int x, int y )
	{	
		// Prepare actioncoords for return to host function
		oActionCoords.Activity = false;
	
		/* Set prev to current coordinates if previous coordinates
			are out of bounds */	
		if ( iPrevX <= 0 )
			iPrevX = x;
		if ( iPrevY <= 0 )
			iPrevY = y;

		double differencex = ( iPrevX - x );
				
		double differencey = ( iPrevY - y );
		
		double distance = sqrt( ( differencex * differencex ) + ( differencey * differencey ) );			
																								
		double angle = atan2( differencey, differencex ) * 180 / fPI;
		
		double pathx = cos( angle * fRAD );
		double pathy = sin( angle * fRAD );											
		
		/* The size which to jump when lightening and darkening */
		double stepsize = oBrush->getRadius ( );
		if ( stepsize < 1.0 ) stepsize = 1.0;
			
		/* Only do something if we have mouse movement! */		
		if ( iPrevX != x or iPrevY != y or !bIsDrawing )
		{													
			if ( sMouseMode == "on" )
			{									
				bIsDrawing = true;								
																								
				// Find ideal antialiasing
				double dx = x - iPrevX;
				double dy = y - iPrevY;
				int aliasing;
				int aliasingx = 2;
				int aliasingy = 2;				
				if ( abs ( dx ) < 2 )
					aliasingx = 2 - dx;
				if ( abs ( dy ) < 2 )
					aliasingy = 2 - dy;
				if ( aliasingx > aliasingy ) aliasing = aliasingx; else aliasing = aliasingy;
				// done finding ideal antialiasing
																		
				for ( double xx = 0.0; xx <= distance; xx += 1 )
				{																
					oBrush->Draw ( canvas, window, x + ( xx * pathx ), y + ( xx * pathy ), aliasing );
				}				
			}		
			else
				bIsDrawing = false;
 		}				
						
		// Draw an outline on the real-time screen															
		if ( oBrush->getSize ( ) > 1 )
		{							
			// Update the coordinated drawn on 
			oActionCoords.Activity = true;	
			
			if ( x >=  ( x + ( distance * pathx ) ) )
			{
				oActionCoords.x = ( int )( x + ( distance * pathx ) - 1 - oBrush->getRadius ( ) ); 
				oActionCoords.w = ( int )x - oActionCoords.x + 2 + oBrush->getRadius ( );
			}
			else
			{
				oActionCoords.x = ( int )x - 1 - oBrush->getRadius ( );
				oActionCoords.w = ( int )( x + ( distance * pathx ) - oActionCoords.x + 2 + oBrush->getRadius ( ) ); 				
			}
			if ( y >=  ( y + ( distance * pathy ) ) )
			{
				oActionCoords.y = ( int )( y + ( distance * pathy ) - 1 - oBrush->getRadius ( ) ); 
				oActionCoords.h = ( int )y - oActionCoords.y + 2 + oBrush->getRadius ( );
			}
			else
			{
				oActionCoords.y = ( int )y - 1 - oBrush->getRadius ( );
				oActionCoords.h = ( int )( y + ( distance * pathy ) - oActionCoords.y + 2 + oBrush->getRadius ( ) ); 				
			}				
			if ( oActionCoords.x < 0 ){ oActionCoords.x = 0; }
			if ( oActionCoords.y < 0 ){ oActionCoords.y = 0; }						
						
			SDL_Rect rect; rect.x = oActionCoords.x; rect.y = oActionCoords.y;
			rect.w = oActionCoords.w; rect.h = oActionCoords.h;				
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &rect, color );		
			
			if ( sMouseMode == "off" )
			{	
				cImageColor tempPixel = oColor; tempPixel.a = 255;
				
				int c, d, dist;																
				
				for ( d = - oBrush->getRadius ( ); d <= oBrush->getRadius ( ); d ++ )
				{
					for ( c = - oBrush->getRadius ( ); c <= oBrush->getRadius ( ); c ++ )
					{	
						dist = ( int ) sqrt ( ( double )( c * c ) + ( double )( d * d ) );
						if ( dist == oBrush->getRadius ( ) & dist > oBrush->getRadius ( ) - 1 )
						{							
							canvas->putPixel ( 
								(int)( x + c ), 
								(int)( y + d ), 
							&tempPixel, window );																
						}						
					}
				}							
			}
		}		
		else
		{									
			// Update the coordinated drawn on 
			oActionCoords.Activity = true;
						
			if ( x >=  ( x + ( distance * pathx ) ) )
			{
				oActionCoords.x = ( int )( x + ( distance * pathx ) ) - 2; 
				oActionCoords.w = ( int )x - oActionCoords.x + 2;
			}
			else
			{
				oActionCoords.x = ( int )x - 2;
				oActionCoords.w = ( int )( x + ( distance * pathx ) ) - oActionCoords.x + 2; 				
			}
			if ( y >=  ( y + ( distance * pathy ) ) )
			{
				oActionCoords.y = ( int )( y + ( distance * pathy ) ) - 2; 
				oActionCoords.h = ( int )y - oActionCoords.y + 2;
			}
			else
			{
				oActionCoords.y = ( int )y - 2;
				oActionCoords.h = ( int )( y + ( distance * pathy ) ) - oActionCoords.y + 2; 				
			}	
			
			SDL_Rect rect; rect.x = oActionCoords.x; rect.y = oActionCoords.y;
			rect.w = oActionCoords.w; rect.h = oActionCoords.h;				
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &rect, color );	
			
			/* The preview pixel */
			if ( sMouseMode == "off" )			
			{
				cImageColor tempPixel = oColor; tempPixel.a = 255;
				canvas->putPixel ( (int)( x ), (int)( y ), &tempPixel, window );																			
			}
		}					
							
		/* Store these coordinates as previous coordinates */
		iPrevX = x; iPrevY = y;
					
		return &oActionCoords;
	}
	
	void cToolBrush::setMouseMode ( string mode )
	{
		if ( mode == "off" or mode == "on" )
			sMouseMode = mode;
		else 
			sMouseMode = "off";
	}

	void cToolBrush::setDrawMode ( string drawmode )
	{
		sDrawMode = drawmode;
	}
	
	string cToolBrush::getName ( )
	{
		return sName;
	}
	
	string cToolBrush::getDrawMode ( )
	{
		return sDrawMode;		
	}
	
	void cToolBrush::setCurrentColor ( cImageColor color )
	{
		oColor = color;
	}
	
	void cToolBrush::sendQuery ( string query )
	{
		//
	}
	
	int cToolBrush::getSize ( )
	{
		return oBrush->getSize ( );
	}
	
	cImageColor cToolBrush::getCurrentColor ( )
	{
		return oColor;
	}
	
	void cToolBrush::setBrush ( cBrush* brush )
	{
		oBrush = brush;
	}

