
	#include "cToolCircle.h"
	
	cToolCircle::cToolCircle ( )
	{
		fPI = 3.14159265358979323846264;
		fRAD = fPI / 180.0f;
		sName = "circle";
		sMouseMode = "off";
		sMousePhase = "none";
		bDoFill = false;
	}
	
	cToolCircle::~cToolCircle ( )
	{
		fPI = 0.0f;
		fRAD = 0.0f;
		sName = "";
		sMouseMode = "";
	}
	
	inline cCoords* cToolCircle::Action ( cImage* canvas, SDL_Surface* window, int x, int y )
	{
		oActionCoords.Activity = false;
	
		if ( sMousePhase == "set" )
		{			
			iX1 = x;
			iY1 = y;			
			oPrevRect.x = 0;
			oPrevRect.y = 0;
			oPrevRect.w = 1;
			oPrevRect.h = 1;	
			sMousePhase = "drag";
		}
		else if ( sMousePhase == "drag" )
		{			
			iX2 = x;
			iY2 = y;
			
			SDL_Rect rect;
			
			rect.x = iX1; 
			rect.w = abs ( iX2 - iX1 );
				
			rect.y = iY1; 
			rect.h = abs ( iY2 - iY1 );		
			
			// Center circle
			rect.w = rect.w * 2;
			rect.h = rect.h * 2;			
			rect.x -= rect.w / 2;
			rect.y -= rect.h / 2;			
			
			// Remove previous
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &oPrevRect, color );
			
			/* Draw the circle with sin and cos */
			int rx, ry;
			
			rx = rect.w / 2; 
			ry = rect.h / 2;								
			
			for ( int c = 0; c < 360; c ++ )
			{
				double dx1 = iX1 + ( cos ( c * fRAD ) * rx );
				double dy1 = iY1 + ( sin ( c * fRAD ) * ry );
				double dx2 = iX1 + ( cos ( ( c + 1 ) * fRAD ) * rx );
				double dy2 = iY1 + ( sin ( ( c + 1 ) * fRAD ) * ry );
				
				double dfx = dx1 - dx2;
				double dfy = dy1 - dy2;
				
				double ang = atan2f ( dfy, dfx ) * 180.0 / fPI;
				
				double distance = sqrt ( ( dfx * dfx ) + ( dfy * dfy ) );								
				
				double pathx = cos ( ang * fRAD );
				double pathy = sin ( ang * fRAD );
				
				for ( ; distance > 0; distance -- )
				{					
					canvas->putPixel ( dx1 + ( pathx * distance ), dy1 + ( pathy * distance ), &oCurrentColor, window );
				}
			}			
			
			// Set return data
			oActionCoords.Activity = true;
			
			oActionCoords.x = oPrevRect.x;
			oActionCoords.y = oPrevRect.y;
			oActionCoords.w = oPrevRect.w;
			oActionCoords.h = oPrevRect.h;
			
			rect.x -= 1; rect.y -= 1; rect.w += 2; rect.h += 2;
			oPrevRect = rect;
							
		}
		else if ( sMousePhase == "apply" )
		{
			SDL_Rect rect;
			
			rect.x = iX1; 
			rect.w = abs ( iX2 - iX1 );
				
			rect.y = iY1; 
			rect.h = abs ( iY2 - iY1 );		
			
			// Center circle
			rect.w = rect.w * 2;
			rect.h = rect.h * 2;			
			rect.x -= rect.w / 2;
			rect.y -= rect.h / 2;									
			
			/* Draw the circle with sin and cos */
			int rx, ry;														
					
			if ( bDoFill )
			{
				rx = ( rect.w / 2 ) - 1; 
				ry = ( rect.h / 2 ) - 1;
				
				for ( int c = 0; c < 360; c ++ )
				{
					double dx1 = iX1 + ( cos ( c * fRAD ) * rx );
					double dy1 = iY1 + ( sin ( c * fRAD ) * ry );
					double dx2 = iX1 + ( cos ( ( c + 1 ) * fRAD ) * rx );
					double dy2 = iY1 + ( sin ( ( c + 1 ) * fRAD ) * ry );
					
					double dfx = dx1 - dx2;
					double dfy = dy1 - dy2;
					
					double ang = atan2f ( dfy, dfx ) * 180.0 / fPI;
					
					double distance = sqrt ( ( dfx * dfx ) + ( dfy * dfy ) );								
					double saveDist = distance;
					
					double pathx = cos ( ang * fRAD );
					double pathy = sin ( ang * fRAD );												
									
					int incrementor;					
					
					if ( iY1 > dy1 + ( pathy * distance ) ) incrementor = 1; else incrementor = -1;
												
					for ( int fy = round ( dy1 + ( pathy * distance ) ); ( int )fy != ( int )iY1 + incrementor; fy += incrementor )
					{									
						oBrush->Draw ( canvas, window, dx1 + ( pathx * distance ), fy, 0 );						
					}
				}			
			
				for ( double a = 0.0; a < 1.5; a += 0.5 )
				{
					rx = ( rect.w / 2 ) - a; 
					ry = ( rect.h / 2 ) - a;
					
					for ( int c = 0; c < 360; c ++ )
					{
						double dx1 = iX1 + ( cos ( c * fRAD ) * rx );
						double dy1 = iY1 + ( sin ( c * fRAD ) * ry );
						double dx2 = iX1 + ( cos ( ( c + 1 ) * fRAD ) * rx );
						double dy2 = iY1 + ( sin ( ( c + 1 ) * fRAD ) * ry );
						
						double dfx = dx1 - dx2;
						double dfy = dy1 - dy2;
						
						double ang = atan2f ( dfy, dfx ) * 180.0 / fPI;
						
						double distance = sqrt ( ( dfx * dfx ) + ( dfy * dfy ) );								
						double saveDist = distance;
						
						double pathx = cos ( ang * fRAD );
						double pathy = sin ( ang * fRAD );												
						
						for ( ; distance > 0; distance -- )
						{																					
							oBrush->Draw ( canvas, window, dx1 + ( pathx * distance ), dy1 + ( pathy * distance ), 2 + distance );
						}
					}
				}
			}
			else
			{
				rx = ( rect.w / 2 );
				ry = ( rect.h / 2 );
				
				for ( int c = 0; c < 360; c ++ )
				{
					double dx1 = iX1 + ( cos ( c * fRAD ) * rx );
					double dy1 = iY1 + ( sin ( c * fRAD ) * ry );
					double dx2 = iX1 + ( cos ( ( c + 1 ) * fRAD ) * rx );
					double dy2 = iY1 + ( sin ( ( c + 1 ) * fRAD ) * ry );
					
					double dfx = dx1 - dx2;
					double dfy = dy1 - dy2;
					
					double ang = atan2f ( dfy, dfx ) * 180.0 / fPI;
					
					double distance = sqrt ( ( dfx * dfx ) + ( dfy * dfy ) );								
					double saveDist = distance;
					
					double pathx = cos ( ang * fRAD );
					double pathy = sin ( ang * fRAD );												
					
					for ( ; distance > 0; distance -- )
					{																					
						oBrush->Draw ( canvas, window, dx1 + ( pathx * distance ), dy1 + ( pathy * distance ), 2 + distance );
					}
				}
			}
			
			sMousePhase = "none";
			
			// Clear temp buffer previous
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &oPrevRect, color );
			
			// Set return data
			oActionCoords.x = -999;
			oActionCoords.y = 0;
			oActionCoords.w = 0;
			oActionCoords.h = 0;
			oActionCoords.Activity = true;
		}	
		return &oActionCoords;
	}
	
	void cToolCircle::sendQuery ( string query )
	{
		if ( query == "toggleFill" )
		{
			if ( bDoFill == false )
			{
				bDoFill = true;
			}
			else
			{
				bDoFill = false;
			}
		}
	}
	
	void cToolCircle::setMouseMode ( string mode )
	{
		if ( mode == "off" or mode == "on" )
		{
			sMouseMode = mode;
			if ( mode == "on" & sMousePhase == "none" )		
				sMousePhase = "set";
			else if ( mode == "on" & sMousePhase == "set" )
				sMousePhase = "drag";
			else if ( mode == "off" & sMousePhase == "drag" )
				sMousePhase = "apply";
		}
		else
		{
			sMouseMode = "off";
		}
	}

	void cToolCircle::setDrawMode ( string drawmode )
	{
		sDrawMode = drawmode;
	}
	
	string cToolCircle::getName ( )
	{
		return sName;
	}
	
	string cToolCircle::getDrawMode ( )
	{
		return sDrawMode;
	}
	
	void cToolCircle::setCurrentColor ( cImageColor color )
	{
		oCurrentColor = color;
	}	
	
	int cToolCircle::getSize ( )
	{	
		return 0;
	}
	
	cImageColor cToolCircle::getCurrentColor ( )
	{
		return oCurrentColor;
	}
	
	void cToolCircle::setBrush ( cBrush* brush )
	{
		oBrush = brush;
	}
