
	#include "cToolClipbrush.h"
	
	cToolClipbrush::cToolClipbrush ( )
	{
		fPI = 3.14159265358979323846264;
		fRAD = fPI / 180.0f;
		sName = "clipbrush";
		sMouseMode = "off";
		sMousePhase = "none";
		bHasBuffer = false;
		bIsMoving = false;
		iPrevX = 0; iPrevY = 0;
		
		if ( SDL_BYTEORDER == SDL_BIG_ENDIAN )
		{
			rmask = 0xff000000;
			gmask = 0x00ff0000;
			bmask = 0x0000ff00;
			amask = 0x000000ff;
		}
		else
		{
			rmask = 0x000000ff;
			gmask = 0x0000ff00;
			bmask = 0x00ff0000;
			amask = 0xff000000;
		}
	}
	
	cToolClipbrush::~cToolClipbrush ( )
	{
		fPI = 0.0f;
		fRAD = 0.0f;
		sName = "";
		sMouseMode = "";
	}
	
	cCoords* cToolClipbrush::Action ( cImage* canvas, SDL_Surface* window, int x, int y )
	{
		oActionCoords.Activity = false;
		
		if ( bHasBuffer )
		{
			if ( sMouseMode == "on" )
			{
				if ( iPrevX != x or iPrevY != y or !bIsMoving )
				{				
					Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
					SDL_FillRect ( window, &oPrevRect, color );
					
					SDL_Rect dest; dest.x = x - ( int )( oClipBuffer->w / 2 ); 
					dest.y = y - ( int )( oClipBuffer->h / 2 ); 
					dest.w = oClipBuffer->w; dest.h = oClipBuffer->h;
					
					for ( int a = 0; a < oClipBuffer->w; a++ )
						for ( int b = 0; b < oClipBuffer->h; b++ )
						{			
							cImageColor color = canvas->getPixel ( a + x - ( int )( oClipBuffer->w / 2 ), b + y - ( int )( oClipBuffer->h / 2 ) );
							cImageColor source = canvas->getPixel ( a, b, oClipBuffer );						
							color.r -= ( int )( ( ( double )( color.r - source.r ) / 255 * source.a ) / 255 * oBrush->getPower ( ) );
							color.g -= ( int )( ( ( double )( color.g - source.g ) / 255 * source.a ) / 255 * oBrush->getPower ( ) );
							color.b -= ( int )( ( ( double )( color.b - source.b ) / 255 * source.a ) / 255 * oBrush->getPower ( ) );
							color.a += ( int )( ( ( double )source.a / 255 * source.a ) / 255 * oBrush->getPower ( ) );
							
							if ( color.r > 255 ) color.r = 255; if ( color.r < 0 ) color.r = 0;
							if ( color.g > 255 ) color.g = 255; if ( color.g < 0 ) color.g = 0;
							if ( color.b > 255 ) color.b = 255; if ( color.b < 0 ) color.b = 0;
							if ( color.a > 255 ) color.a = 255; if ( color.a < 0 ) color.a = 0;			
							canvas->putPixel ( x + a - ( int )( oClipBuffer->w / 2 ), y + b - ( int )( oClipBuffer->h / 2 ), &color );	
						}
					
					oActionCoords.Activity = true;
					oActionCoords.x = dest.x;
					oActionCoords.y = dest.y;
					oActionCoords.w = dest.w;
					oActionCoords.h = dest.h;
					
					bIsMoving = true;			
					iPrevX = x; iPrevY = y;									
				}
			}
			else
			{
				Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
				SDL_FillRect ( window, &oPrevRect, color );
				
				SDL_Rect dest; 
				dest.x = x - ( int )( oClipBuffer->w / 2 ); 
				dest.y = y - ( int )( oClipBuffer->h / 2 ); 
				dest.w = oClipBuffer->w; dest.h = oClipBuffer->h;
				
				cImageColor tempCol;
				
				for ( int a = 0; a < oClipBuffer->w; a++ )
				{
					for ( int b = 0; b < oClipBuffer->h; b++ )
					{
						tempCol = canvas->getPixel ( a, b, oClipBuffer );
						canvas->putPixel ( x + a - ( int )( oClipBuffer->w / 2 ), y + b - ( int )( oClipBuffer->h / 2 ), &tempCol, window );
					}
				}
				
				oActionCoords.Activity = true;
				
				oActionCoords.x = oPrevRect.x;
				oActionCoords.y = oPrevRect.y;
				oActionCoords.w = oPrevRect.w;
				oActionCoords.h = oPrevRect.h;	
				
				oPrevRect = dest;		
				
				bIsMoving = false;		
				iPrevX = x; iPrevY = y;				
			}			
		}
		else
		{			
			if ( sMousePhase == "set" )
			{			
				iX1 = x;
				iY1 = y;			
				oPrevRect.x = 0;
				oPrevRect.y = 0;
				oPrevRect.w = 1;
				oPrevRect.h = 1;	
				sMousePhase = "drag";
			}
			else if ( sMousePhase == "drag" )
			{			
				// New values
				iX2 = x;
				iY2 = y;
				
				SDL_Rect rect;
				if ( iX1 < iX2 )
				{
					rect.x = iX1; 			
					rect.w = iX2 - iX1;
				}
				else
				{
					rect.x = iX2; 				
					rect.w = iX1 - iX2;
				}
				if ( iY1 < iY2 )
				{
					rect.y = iY1; 				
					rect.h = iY2 - iY1;
				}
				else
				{
					rect.y = iY2; 				
					rect.h = iY1 - iY2;
				}
											
				Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
				SDL_FillRect ( window, &oPrevRect, color );
				
				Uint32 col = SDL_MapRGBA ( window->format, oCurrentColor.r, oCurrentColor.g, oCurrentColor.b, oCurrentColor.a );
				SDL_FillRect( window, &rect, col );
				
				// Make outline selector
				if ( rect.w > 2 & rect.h > 2 )
				{
					SDL_Rect rect2; rect2.x = rect.x + 1; rect2.y = rect.y + 1; rect2.w = rect.w - 2; rect2.h = rect.h - 2;
					SDL_FillRect( window, &rect2, color );
				}
				
				
				// Set return data
				oActionCoords.Activity = true;
				
				oActionCoords.x = oPrevRect.x;
				oActionCoords.y = oPrevRect.y;
				oActionCoords.w = oPrevRect.w;
				oActionCoords.h = oPrevRect.h;
				
				oPrevRect = rect;			
			}
			else if ( sMousePhase == "apply" )
			{						
				SDL_Rect rect;
				if ( iX1 < iX2 )
				{
					rect.x = iX1; 
					rect.w = iX2 - iX1;
				}
				else
				{
					rect.x = iX2; 
					rect.w = iX1 - iX2;
				}
				if ( iY1 < iY2 )
				{
					rect.y = iY1; 
					rect.h = iY2 - iY1;
				}
				else
				{
					rect.y = iY2; 
					rect.h = iY1 - iY2;
				}
				
				/* Remove prev */
				Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
				SDL_FillRect ( window, &oPrevRect, color );
				
				/* Copy */								
				
				cImageColor tempCol;	
				
				SDL_Rect destrect; destrect.x = 0; destrect.y = 0; destrect.w = rect.w; destrect.h = rect.h;
				oClipBuffer = SDL_CreateRGBSurface ( SDL_SWSURFACE, rect.w, rect.h, 32, rmask, gmask, bmask, amask );	
				
				for ( int a = rect.x; a < rect.x + rect.w; a++ )
					for ( int b = rect.y; b < rect.y + rect.h; b++ )
					{
						tempCol = canvas->getPixel ( a, b );						
						canvas->putPixel ( a - rect.x, b - rect.y, &tempCol, oClipBuffer );								
					}
				bHasBuffer = true;			
				/* Done copying */
				
				sMousePhase = "none";		
				
				// Set return data
				oActionCoords.Activity = true;
				oActionCoords.x = rect.x;
				oActionCoords.y = rect.y;
				oActionCoords.w = rect.w;
				oActionCoords.h = rect.h;	
				
				oPrevRect = rect;					
			}					
		}		
		
		return &oActionCoords;	
	}
	
	void cToolClipbrush::setMouseMode ( string mode )
	{
		if ( mode == "off" or mode == "on" )
		{
			sMouseMode = mode;
			if ( !bHasBuffer )
			{
				if ( mode == "on" & sMousePhase == "none" )		
					sMousePhase = "set";
				else if ( mode == "on" & sMousePhase == "set" )
					sMousePhase = "drag";
				else if ( mode == "off" & sMousePhase == "drag" )
					sMousePhase = "apply";			
			}			
		}
		else
		{
			sMouseMode = "off";
		}
	}

	void cToolClipbrush::setDrawMode ( string drawmode )
	{
		sDrawMode = drawmode;
	}
	
	string cToolClipbrush::getName ( )
	{
		return sName;
	}
	
	string cToolClipbrush::getDrawMode ( )
	{
		return sDrawMode;
	}
	
	void cToolClipbrush::setCurrentColor ( cImageColor color )
	{
		oCurrentColor = color;
	}
	
	void cToolClipbrush::sendQuery ( string query )
	{		
		if ( query == "clearBrushBuffer" )
		{
			bHasBuffer = false;			
		}
		else if ( query == "flipx" ) 
		{
			flipX ( );
		}
		else if ( query == "flipy" ) 
		{
			flipY ( );
		}
		else if ( query == "rotate" ) 
		{
			rotate ( );
		}
	}
	
	int cToolClipbrush::getSize ( )
	{	
		return 0;
	}
	
	cImageColor cToolClipbrush::getCurrentColor ( )
	{
		return oCurrentColor;
	}

	void cToolClipbrush::flipX ( )
	{		
		for ( int y = 0; y < oClipBuffer->h; y++ )
		{
			for ( int x = 0; x < oClipBuffer->w / 2; x++ )
			{
				cImageColor pixel1 = oImage.getPixel ( x, y, oClipBuffer );
				cImageColor pixel2 = oImage.getPixel ( oClipBuffer->w - x - 1, y, oClipBuffer );
				oImage.putPixel ( x, y, &pixel2, oClipBuffer );
				oImage.putPixel ( oClipBuffer->w - x - 1, y, &pixel1, oClipBuffer );
			}
		}
	}
	
	void cToolClipbrush::flipY ( )
	{		
		for ( int x = 0; x < oClipBuffer->w; x++ )
		{
			for ( int y = 0; y < oClipBuffer->h / 2; y++ )
			{
				cImageColor pixel1 = oImage.getPixel ( x, y, oClipBuffer );
				cImageColor pixel2 = oImage.getPixel ( x, oClipBuffer->h - y - 1, oClipBuffer );
				oImage.putPixel ( x, y, &pixel2, oClipBuffer );
				oImage.putPixel ( x, oClipBuffer->h - y - 1, &pixel1, oClipBuffer );
			}			
		}
	}
	
 	void cToolClipbrush::rotate ( )
	{		
		SDL_Surface* tempSurf = SDL_CreateRGBSurface ( SDL_SWSURFACE, oClipBuffer->h, oClipBuffer->w, 32, rmask, gmask, bmask, amask );
		
		for ( int y = 0; y < oClipBuffer->h; y++ )
		{							
			for ( int x = 0; x < oClipBuffer->w; x++ )
			{		
				cImageColor pixel = oImage.getPixel ( x, y, oClipBuffer );
				oImage.putPixel ( oClipBuffer->h - y - 1, x, &pixel, tempSurf );				
			}
		}
		
		oClipBuffer = SDL_CreateRGBSurface ( SDL_SWSURFACE, tempSurf->w, tempSurf->h, 32, rmask, gmask, bmask, amask );
		
		for ( int y = 0; y < tempSurf->h; y++ )
		{
			for ( int x = 0; x < tempSurf->w; x++ )
			{
				cImageColor pixel = oImage.getPixel ( x, y, tempSurf );
				oImage.putPixel ( x, y, &pixel, oClipBuffer );
			}
		}
		SDL_FreeSurface ( tempSurf );
	}
	
	void cToolClipbrush::setBrush ( cBrush* brush )
	{
		oBrush = brush;
	}
