
	#include "cToolLine.h"
	
	cToolLine::cToolLine ( )
	{
		fPI = 3.14159265358979323846264;
		fRAD = fPI / 180.0f;
		sName = "line";
		sMouseMode = "off";
		sMousePhase = "none";
		
		oPrevRect.w = 1;
		oPrevRect.h = 1;
	}
	
	cToolLine::~cToolLine ( )
	{
		fPI = 0.0f;
		fRAD = 0.0f;
		sName = "";
		sMouseMode = "";
	}
	
	cCoords* cToolLine::Action ( cImage* canvas, SDL_Surface* window, int x, int y )
	{
		oActionCoords.Activity = false;
					
		if ( sMousePhase == "set" )
		{			
			iX1 = x;
			iY1 = y;					
			sMousePhase = "drag";
		}
		else if ( sMousePhase == "drag" )
		{			
			// New values
			iX2 = x;
			iY2 = y;
			
			SDL_Rect rect;
			if ( iX1 < iX2 )
			{
				rect.x = iX1; 			
				rect.w = iX2 - iX1;
			}
			else
			{
				rect.x = iX2; 				
				rect.w = iX1 - iX2;
			}
			if ( iY1 < iY2 )
			{
				rect.y = iY1; 				
				rect.h = iY2 - iY1;
			}
			else
			{
				rect.y = iY2; 				
				rect.h = iY1 - iY2;
			}			
			
			rect.x -= 1; rect.y -= 1;
			rect.w += 2; rect.h += 2;
										
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &oPrevRect, color );						
									
			double dx = ( iX2 - iX1 );
			double dy = ( iY2 - iY1 );
			double length = sqrt ( ( dx * dx ) + ( dy * dy ) );
			double sx = dx / length;
			double sy = dy / length;
			double ax = iX1;
			double ay = iY1;
						
			canvas->putPixel ( ( int )ax, ( int )ay, &oColor, window );
			
			for ( double inc = 0; inc <= length; inc ++ )
			{								
				// Error correction
				if ( inc == 0 )
				{
					if ( iX1 > iX2 )
					{
						ax += 1;
					}	
					if ( iY1 > iY2 )
					{
						ay += 1;
					}									
				}
				else
					canvas->putPixel ( ( int )ax, ( int )ay, &oColor, window );
				
				ax += sx;
				ay += sy;				
			}
			
			
			// Set return data
			oActionCoords.Activity = true;
			
			oActionCoords.x = oPrevRect.x;
			oActionCoords.y = oPrevRect.y;
			oActionCoords.w = oPrevRect.w;
			oActionCoords.h = oPrevRect.h;
			
			oPrevRect = rect;									
		}
		else if ( sMousePhase == "apply" )
		{						
			SDL_Rect rect;
			
			int brushradius = oBrush->getRadius ( );
			int brushsize = oBrush->getSize ( );
			
			if ( iX1 < iX2 )
			{
				rect.x = iX1 - brushradius; 
				rect.w = iX2 - iX1 + brushsize;
			}
			else
			{
				rect.x = iX2 - brushradius; 
				rect.w = iX1 - iX2 + brushsize;
			}
			if ( iY1 < iY2 )
			{
				rect.y = iY1 - brushradius; 
				rect.h = iY2 - iY1 + brushsize;
			}
			else
			{
				rect.y = iY2 - brushradius; 
				rect.h = iY1 - iY2 + brushsize;
			}
			
			// Give some leverage
			rect.x -= 1; rect.y -= 1; 
			rect.w += 2; rect.h += 2;
			
			/* Remove prev */
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &oPrevRect, color );					
			
			double dx = ( iX2 - iX1 );
			double dy = ( iY2 - iY1 );
			double length = sqrt ( ( dx * dx ) + ( dy * dy ) );
			double sx = dx / length;
			double sy = dy / length;
			double ax = iX1;
			double ay = iY1;
								
			oBrush->Draw ( canvas, window, ax, ay, length );
			
			// Find ideal antialiasing
			int aliasing;
			int aliasingx = 2;
			int aliasingy = 2;			
			if ( abs ( dx ) < 2 )
				aliasingx = ( int )( 2 - dx );
			if ( abs ( dy ) < 2 )
				aliasingy = ( int )( 2 - dy );
			if ( aliasingx > aliasingy ) aliasing = aliasingx; else aliasing = aliasingy;
			// done finding ideal antialiasing
			
			for ( double inc = 0; inc <= length; inc ++ )
			{													
				// Error correction
				if ( inc == 0 )
				{
					if ( iX1 > iX2 )
					{
						ax += 1;
					}	
					if ( iY1 > iY2 )
					{
						ay += 1;
					}									
				}
				else
					oBrush->Draw ( canvas, window, ax, ay, abs ( aliasing ) );			
											
				ax += sx;
				ay += sy;				
			}			
			
			sMousePhase = "none";		
			
			// Set return data
			oActionCoords.Activity = true;
 			oActionCoords.x = rect.x;
			oActionCoords.y = rect.y;
			oActionCoords.w = rect.w;
			oActionCoords.h = rect.h;						
		}	
		else
		{		
			if ( oPrevRect.x > window->w || oPrevRect.x < 0 )
				oPrevRect.x = x;
			if ( oPrevRect.y > window->h || oPrevRect.y < 0 )
				oPrevRect.y = y;							
				
			// Update the coordinated drawn on 
			oActionCoords.Activity = true;
						
			if ( oPrevRect.x >=  x )
			{
				oActionCoords.x = x - 1; 
				oActionCoords.w = oPrevRect.x - x + 2;
			}
			else
			{
				oActionCoords.x = ( int )oPrevRect.x - 1;
				oActionCoords.w = x - oPrevRect.x + 2; 				
			}
			if ( oPrevRect.y >=  y )
			{
				oActionCoords.y = y - 1; 
				oActionCoords.h = oPrevRect.y - y + 2;
			}
			else
			{
				oActionCoords.y = ( int )oPrevRect.y - 1;
				oActionCoords.h = ( int ) y - oPrevRect.y + 2; 								
			}				
			
			SDL_Rect rect; rect.x = oActionCoords.x; rect.y = oActionCoords.y;
			rect.w = oActionCoords.w; rect.h = oActionCoords.h;				
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &rect, color );							
			
			canvas->putPixel ( x, y, &oColor, window );												
			
			oPrevRect.x = x; oPrevRect.y = y;
			oPrevRect.w = 1; oPrevRect.h = 1;						
		}						
		return &oActionCoords;	
	}
	
	void cToolLine::setMouseMode ( string mode )
	{
		if ( mode == "off" or mode == "on" )
		{
			sMouseMode = mode;
			if ( mode == "on" & sMousePhase == "none" )		
				sMousePhase = "set";
			else if ( mode == "on" & sMousePhase == "set" )
				sMousePhase = "drag";
			else if ( mode == "off" & sMousePhase == "drag" )
				sMousePhase = "apply";
		}
		else
		{
			sMouseMode = "off";
		}
	}

	void cToolLine::setDrawMode ( string drawmode )
	{
		sDrawMode = drawmode;
	}
	
	string cToolLine::getName ( )
	{
		return sName;
	}
	
	string cToolLine::getDrawMode ( )
	{
		return sDrawMode;
	}
	
	void cToolLine::setCurrentColor ( cImageColor color )
	{
		oColor = color;
	}
	
	void cToolLine::sendQuery ( string query )
	{		
	}
	
	int cToolLine::getSize ( )
	{	
		return 0;
	}
	
	cImageColor cToolLine::getCurrentColor ( )
	{
		return oColor;
	}
	
	void cToolLine::setBrush ( cBrush* brush )
	{
		oBrush = brush;
	}

