
	#include "cToolRectangle.h"
	
	cToolRectangle::cToolRectangle ( )
	{
		fPI = 3.14159265358979323846264;
		fRAD = fPI / 180.0f;
		sName = "rectangle";
		sMouseMode = "off";
		sMousePhase = "none";
		bDoFill = false;
	}
	
	cToolRectangle::~cToolRectangle ( )
	{
		fPI = 0.0f;
		fRAD = 0.0f;
		sName = "";
		sMouseMode = "";
	}
	
	cCoords* cToolRectangle::Action ( cImage* canvas, SDL_Surface* window, int x, int y )
	{
		oActionCoords.Activity = false;
					
		if ( sMousePhase == "set" )
		{			
			iX1 = x;
			iY1 = y;			
			oPrevRect.x = 0;
			oPrevRect.y = 0;
			oPrevRect.w = 1;
			oPrevRect.h = 1;	
			sMousePhase = "drag";
		}
		else if ( sMousePhase == "drag" )
		{			
			// New values
			iX2 = x;
			iY2 = y;
			
			SDL_Rect rect;
			if ( iX1 < iX2 )
			{
				rect.x = iX1; 			
				rect.w = iX2 - iX1;
			}
			else
			{
				rect.x = iX2; 				
				rect.w = iX1 - iX2;
			}
			if ( iY1 < iY2 )
			{
				rect.y = iY1; 				
				rect.h = iY2 - iY1;
			}
			else
			{
				rect.y = iY2; 				
				rect.h = iY1 - iY2;
			}
										
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &oPrevRect, color );
			
			Uint32 col = SDL_MapRGBA ( window->format, oCurrentColor.r, oCurrentColor.g, oCurrentColor.b, oCurrentColor.a );
			SDL_FillRect( window, &rect, col );																	
			
			
			// Set return data
			oActionCoords.Activity = true;
			
			oActionCoords.x = oPrevRect.x;
			oActionCoords.y = oPrevRect.y;
			oActionCoords.w = oPrevRect.w;
			oActionCoords.h = oPrevRect.h;
			
			oPrevRect = rect;
		}
		else if ( sMousePhase == "apply" )
		{						
			SDL_Rect rect;
			if ( iX1 < iX2 )
			{
				rect.x = iX1; 
				rect.w = iX2 - iX1;
			}
			else
			{
				rect.x = iX2; 
				rect.w = iX1 - iX2;
			}
			if ( iY1 < iY2 )
			{
				rect.y = iY1; 
				rect.h = iY2 - iY1;
			}
			else
			{
				rect.y = iY2; 
				rect.h = iY1 - iY2;
			}
			
			/* Remove prev */
			
			Uint32 color = SDL_MapRGBA ( window->format, 0, 0, 0, 0 );
			SDL_FillRect ( window, &oPrevRect, color );
			
			if ( bDoFill )
			{
				for ( int x = rect.x; x <= rect.x + rect.w - 1; x++ )
				{
					for ( int y = rect.y; y <= rect.y + rect.h - 1; y++ )
					{
						oBrush->Draw ( canvas, window, x, y, 0 );
					}
				}
			}
			else
			{
				for ( int x = rect.x; x < rect.x + rect.w; x++ )
				{					
					oBrush->Draw ( canvas, window, x, rect.y, 0 );
					oBrush->Draw ( canvas, window, x, rect.y + rect.h - 1, 0 );
				}
				for ( int y = rect.y; y < rect.y + rect.h; y++ )
				{
					oBrush->Draw ( canvas, window, rect.x, y, 0 );
					oBrush->Draw ( canvas, window, rect.x + rect.w - 1, y, 0 );
				}
			}
			
			sMousePhase = "none";		
			
			// Set return data
			oActionCoords.Activity = true;
 			oActionCoords.x = rect.x - oBrush->getRadius ( );
			oActionCoords.y = rect.y - oBrush->getRadius ( );
			oActionCoords.w = rect.w + ( int )oBrush->getSize ( );
			oActionCoords.h = rect.h + ( int )oBrush->getSize ( );						
		}					
		
		return &oActionCoords;	
	}
	
	void cToolRectangle::setMouseMode ( string mode )
	{
		if ( mode == "off" or mode == "on" )
		{
			sMouseMode = mode;
			if ( mode == "on" & sMousePhase == "none" )		
				sMousePhase = "set";
			else if ( mode == "on" & sMousePhase == "set" )
				sMousePhase = "drag";
			else if ( mode == "off" & sMousePhase == "drag" )
				sMousePhase = "apply";
		}
		else
		{
			sMouseMode = "off";
		}
	}

	void cToolRectangle::setDrawMode ( string drawmode )
	{
		sDrawMode = drawmode;
	}
	
	string cToolRectangle::getName ( )
	{
		return sName;
	}
	
	string cToolRectangle::getDrawMode ( )
	{
		return sDrawMode;
	}
	
	void cToolRectangle::setCurrentColor ( cImageColor color )
	{
		oCurrentColor = color;
	}
	
	void cToolRectangle::sendQuery ( string query )
	{						
		if ( query == "toggleFill" )
		{
			if ( bDoFill == false )
				bDoFill = true;
			else
				bDoFill = false;
		}
	}
	
	int cToolRectangle::getSize ( )
	{	
		return 0;
	}
	
	cImageColor cToolRectangle::getCurrentColor ( )
	{
		return oCurrentColor;
	}

	void cToolRectangle::setBrush ( cBrush* brush )
	{
		oBrush = brush;
	}

